# -*- coding: utf-8
"""This module calculates XC kernels for response function calculations.
"""
from __future__ import print_function

import numpy as np
from scipy.special import spherical_jn

from gpaw.xc import XC
from gpaw.sphere.lebedev import weight_n, R_nv
from gpaw.io.tar import Reader
from gpaw.wavefunctions.pw import PWDescriptor
from gpaw.spherical_harmonics import Yarr

from ase.utils.timing import Timer
from ase.units import Bohr, Ha


def get_xc_kernel(pd, chi0, functional='ALDA', kernel='density',
                  rshe=0.99,
                  chi0_wGG=None,
                  fxc_scaling=None,
                  density_cut=None,
                  spinpol_cut=None):
    """
    Factory function that calls the relevant functions below
    """

    if kernel == 'density':
        return get_density_xc_kernel(pd, chi0, functional=functional,
                                     rshe=rshe,
                                     chi0_wGG=chi0_wGG,
                                     density_cut=density_cut)
    elif kernel in ['+-', '-+']:
        # Currently only collinear adiabatic xc kernels are implemented
        # for which the +- and -+ kernels are the same
        return get_transverse_xc_kernel(pd, chi0, functional=functional,
                                        rshe=rshe,
                                        chi0_wGG=chi0_wGG,
                                        fxc_scaling=fxc_scaling,
                                        density_cut=density_cut,
                                        spinpol_cut=spinpol_cut)
    else:
        raise ValueError('%s kernels not implemented' % kernel)


def get_density_xc_kernel(pd, chi0, functional='ALDA',
                          rshe=0.99,
                          chi0_wGG=None,
                          density_cut=None):
    """
    Density-density xc kernels
    Factory function that calls the relevant functions below
    """

    calc = chi0.calc
    fd = chi0.fd
    nspins = len(calc.density.nt_sG)
    assert nspins == 1

    if functional[0] == 'A':
        # Standard adiabatic kernel
        print('Calculating %s kernel' % functional, file=fd)
        Kcalc = AdiabaticDensityKernelCalculator(fd, chi0.world,
                                                 rshe=rshe,
                                                 ecut=chi0.ecut,
                                                 density_cut=density_cut)
        Kxc_sGG = np.array([Kcalc(pd, calc, functional)])
    elif functional[0] == 'r':
        # Renormalized kernel
        print('Calculating %s kernel' % functional, file=fd)
        Kxc_sGG = calculate_renormalized_kernel(pd, calc, functional, fd)
    elif functional[:2] == 'LR':
        print('Calculating LR kernel with alpha = %s' % functional[2:],
              file=fd)
        Kxc_sGG = calculate_lr_kernel(pd, calc, alpha=float(functional[2:]))
    elif functional == 'DM':
        print('Calculating DM kernel', file=fd)
        Kxc_sGG = calculate_dm_kernel(pd, calc)
    elif functional == 'Bootstrap':
        print('Calculating Bootstrap kernel', file=fd)
        Kxc_sGG = get_bootstrap_kernel(pd, chi0, chi0_wGG, fd)
    else:
        raise ValueError('density-density %s kernel not'
                         + ' implemented' % functional)

    return Kxc_sGG[0]


def get_transverse_xc_kernel(pd, chi0, functional='ALDA_x',
                             rshe=0.99,
                             chi0_wGG=None,
                             fxc_scaling=None,
                             density_cut=None,
                             spinpol_cut=None):
    """ +-/-+ xc kernels
    Currently only collinear ALDA kernels are implemented
    Factory function that calls the relevant functions below
    """

    calc = chi0.calc
    fd = chi0.fd
    nspins = len(calc.density.nt_sG)
    assert nspins == 2

    if functional in ['ALDA_x', 'ALDA_X', 'ALDA']:
        # Adiabatic kernel
        print("Calculating transverse %s kernel" % functional, file=fd)
        Kcalc = AdiabaticTransverseKernelCalculator(fd, chi0.world,
                                                    rshe=rshe,
                                                    ecut=chi0.ecut,
                                                    density_cut=density_cut,
                                                    spinpol_cut=spinpol_cut)
    else:
        raise ValueError("%s spin kernel not implemented" % functional)

    Kxc_GG = Kcalc(pd, calc, functional)

    if fxc_scaling is not None:
        assert isinstance(fxc_scaling[0], bool)
        if fxc_scaling[0]:
            if fxc_scaling[1] is None:
                fxc_scaling[1] = find_Goldstone_scaling(pd, chi0,
                                                        chi0_wGG, Kxc_GG)

            assert isinstance(fxc_scaling[1], float)
            Kxc_GG *= fxc_scaling[1]

    return Kxc_GG


def find_Goldstone_scaling(pd, chi0, chi0_wGG, Kxc_GG):
    """ Find a scaling of the kernel to move the magnon peak to omeaga=0. """
    # q should be gamma - scale to hit Goldstone
    assert pd.kd.gamma

    fd = chi0.fd
    omega_w = chi0.omega_w
    wgs = np.abs(omega_w).argmin()

    if not np.allclose(omega_w[wgs], 0., rtol=1.e-8):
        raise ValueError("Frequency grid needs to include"
                         + " omega=0. to allow Goldstone scaling")

    fxcs = 1.
    print("Finding rescaling of kernel to fulfill the Goldstone theorem",
          file=fd)

    world = chi0.world
    # Only one rank, rgs, has omega=0 and finds rescaling
    nw = len(omega_w)
    mynw = (nw + world.size - 1) // world.size
    rgs, mywgs = wgs // mynw, wgs % mynw
    fxcsbuf = np.empty(1, dtype=float)
    if world.rank == rgs:
        chi0_GG = chi0_wGG[mywgs]
        chi_GG = np.dot(np.linalg.inv(np.eye(len(chi0_GG)) -
                                      np.dot(chi0_GG, Kxc_GG * fxcs)),
                        chi0_GG)
        # Scale so that kappaM=0 in the static limit (omega=0)
        kappaM = (chi0_GG[0, 0] / chi_GG[0, 0]).real
        # If kappaM > 0, increase scaling (recall: kappaM ~ 1 - Kxc Re{chi_0})
        scaling_incr = 0.1 * np.sign(kappaM)
        while abs(kappaM) > 1.e-7 and abs(scaling_incr) > 1.e-7:
            fxcs += scaling_incr
            if fxcs <= 0.0 or fxcs >= 10.:
                raise Exception('Found an invalid fxc_scaling of %.4f' % fxcs)

            chi_GG = np.dot(np.linalg.inv(np.eye(len(chi0_GG)) -
                                          np.dot(chi0_GG, Kxc_GG * fxcs)),
                            chi0_GG)
            kappaM = (chi0_GG[0, 0] / chi_GG[0, 0]).real

            # If kappaM changes sign, change sign and refine increment
            if np.sign(kappaM) != np.sign(scaling_incr):
                scaling_incr *= -0.2
        fxcsbuf[:] = fxcs

    # Broadcast found rescaling
    world.broadcast(fxcsbuf, rgs)
    fxcs = fxcsbuf[0]

    return fxcs


class AdiabaticKernelCalculator:
    """ Adiabatic kernels with PAW """

    def __init__(self, fd, world, rshe=0.99, ecut=None):
        """
        rshe : float or None
            Expand kernel in real spherical harmonics inside augmentation
            spheres. If None, the kernel will be calculated without
            augmentation. The value of rshe (0<rshe<1) sets a convergence
            criteria for the expansion in real spherical harmonics.
        """

        self.fd = fd
        self.world = world
        self.ecut = ecut

        self.permitted_functionals = []
        self.functional = None

        self.rshe = rshe is not None

        if self.rshe:
            self.rshecc = rshe
            self.dfSns_g = None
            self.dfSns_gL = None
            self.dfmask_g = None
            self.rsheconvmin = None

            self.rsheL_M = None

    def __call__(self, pd, calc, functional):
        assert functional in self.permitted_functionals
        self.functional = functional
        add_fxc = self.add_fxc  # class methods not within the scope of call

        vol = pd.gd.volume
        npw = pd.ngmax

        if self.rshe:
            nt_sG = calc.density.nt_sG
            gd, lpd = pd.gd, pd

            print("\tCalculating fxc on real space grid using smooth density",
                  file=self.fd)
            fxc_G = np.zeros(np.shape(nt_sG[0]))
            add_fxc(gd, nt_sG, fxc_G)
        else:
            print("\tFinding all-electron density", file=self.fd)
            n_sG, gd = calc.density.get_all_electron_density(atoms=calc.atoms,
                                                             gridrefinement=1)
            qd = pd.kd
            lpd = PWDescriptor(self.ecut, gd, complex, qd,
                               gammacentered=pd.gammacentered)

            print("\tCalculating fxc on real space grid using"
                  + " all-electron density", file=self.fd)
            fxc_G = np.zeros(np.shape(n_sG[0]))
            add_fxc(gd, n_sG, fxc_G)

        print("\tFourier transforming into reciprocal space", file=self.fd)
        nG = gd.N_c
        nG0 = nG[0] * nG[1] * nG[2]

        tmp_g = np.fft.fftn(fxc_G) * vol / nG0

        Kxc_GG = np.zeros((npw, npw), dtype=complex)
        for iG, iQ in enumerate(lpd.Q_qG[0]):
            iQ_c = (np.unravel_index(iQ, nG) + nG // 2) % nG - nG // 2
            for jG, jQ in enumerate(lpd.Q_qG[0]):
                jQ_c = (np.unravel_index(jQ, nG) + nG // 2) % nG - nG // 2
                ijQ_c = (iQ_c - jQ_c)
                if (abs(ijQ_c) < nG // 2).all():
                    Kxc_GG[iG, jG] = tmp_g[tuple(ijQ_c)]

        if self.rshe:
            print("\tCalculating PAW corrections to the kernel", file=self.fd)

            G_Gv = pd.get_reciprocal_vectors()
            R_av = calc.atoms.positions / Bohr
            setups = calc.wfs.setups
            D_asp = calc.density.D_asp

            KxcPAW_GG = np.zeros_like(Kxc_GG)

            # Distribute correction and find dG
            nGpr = (npw + self.world.size - 1) // self.world.size
            Ga = min(self.world.rank * nGpr, npw)
            Gb = min(Ga + nGpr, npw)
            myG_G = range(Ga, Gb)
            dG_GGv = np.zeros((len(myG_G), npw, 3))
            for v in range(3):
                dG_GGv[:, :, v] = np.subtract.outer(G_Gv[myG_G, v], G_Gv[:, v])

            # Find length of dG and the normalized dG
            dG_GG = np.linalg.norm(dG_GGv, axis=2)
            dGn_GGv = np.zeros_like(dG_GGv)
            mask0 = np.where(dG_GG != 0.)
            dGn_GGv[mask0] = dG_GGv[mask0] / dG_GG[mask0][:, np.newaxis]

            for a, setup in enumerate(setups):
                n_qg = setup.xc_correction.n_qg
                nt_qg = setup.xc_correction.nt_qg
                nc_g = setup.xc_correction.nc_g
                nct_g = setup.xc_correction.nct_g

                D_sp = D_asp[a]
                B_pqL = setup.xc_correction.B_pqL
                D_sLq = np.inner(D_sp, B_pqL.T)
                nspins = len(D_sp)

                n_sLg = np.dot(D_sLq, n_qg)
                nt_sLg = np.dot(D_sLq, nt_qg)

                # Add core density
                n_sLg[:, 0] += np.sqrt(4. * np.pi) / nspins * nc_g
                nt_sLg[:, 0] += np.sqrt(4. * np.pi) / nspins * nct_g

                # Calculate dfxc
                # Please note: Using the radial grid descriptor with add_fxc
                # might give problems beyond ALDA
                Y_nL = setup.xc_correction.Y_nL
                rgd = setup.xc_correction.rgd
                f_g = rgd.zeros()
                ft_g = rgd.zeros()
                df_ng = np.array([rgd.zeros() for n in range(len(R_nv))])
                for n, Y_L in enumerate(Y_nL):
                    f_g[:] = 0.
                    n_sg = np.dot(Y_L, n_sLg)
                    add_fxc(rgd, n_sg, f_g)

                    ft_g[:] = 0.
                    nt_sg = np.dot(Y_L, nt_sLg)
                    add_fxc(rgd, nt_sg, ft_g)

                    df_ng[n, :] = f_g - ft_g

                # Expand angular part of dfxc in real spherical harmonics.
                # Note that the Lebedev quadrature, which is used to calculate
                # the expansion coefficients, is exact to order l=11. This
                # implies that functions containing angular components l<=5 can
                # be expanded exactly.
                L_L = []
                l_L = []
                nL = min(Y_nL.shape[1], 36)
                # The convergence of the expansion is tracked through the
                # surface norm square of df
                ng = self._initialize_rshe_convcrit(df_ng, nL)
                # Integrate only r-values inside augmentation sphere
                df_ng = df_ng[:, :ng]
                r_g = rgd.r_g[:ng]
                dv_g = rgd.dv_g[:ng]

                # Add real spherical harmonics to fulfill convergence criteria.
                df_gL = np.zeros((ng, nL))
                l = 0
                while self.rsheconvmin < self.rshecc:
                    if l > int(np.sqrt(nL) - 1):
                        raise Exception('Could not expand %.f of' % self.rshecc
                                        + ' PAW correction to atom %d in ' % a
                                        + 'real spherical harmonics up to '
                                        + 'order l=%d' % int(np.sqrt(nL) - 1))

                    L_L += range(l**2, l**2 + 2 * l + 1)
                    l_L += [l] * (l * 2 + 1)

                    self._add_rshe_coefficients(df_ng, df_gL, Y_nL, l)
                    self._evaluate_rshe_convergence(df_gL)

                    print('\tAt least a fraction of '
                          + '%.8f' % self.rsheconvmin
                          + ' of the PAW correction to atom %d could be ' % a
                          + 'expanded in spherical harmonics up to l=%d' % l,
                          file=self.fd)
                    l += 1

                # Filter away unused (l,m)-coefficients
                L_M = np.where(self.rsheL_M)[0]
                l_M = [l_L[L] for L in L_M]

                # Expand plane wave
                nM = len(L_M)
                (r_gMGG, l_gMGG,
                 dG_gMGG) = [a.reshape(ng, nM, *dG_GG.shape)
                             for a in np.meshgrid(r_g, l_M, dG_GG.flatten(),
                                                  indexing='ij')]
                j_gMGG = spherical_jn(l_gMGG, dG_gMGG * r_gMGG)  # Slow step
                Y_MGG = Yarr(L_M, dGn_GGv)
                ii_MK = (-1j) ** np.repeat(l_M,
                                           np.prod(dG_GG.shape))
                ii_MGG = ii_MK.reshape((nM, *dG_GG.shape))

                # Perform integration
                coefatomR_GG = np.exp(-1j * np.inner(dG_GGv, R_av[a]))
                coefatomang_MGG = ii_MGG * Y_MGG
                coefatomrad_MGG = np.tensordot(j_gMGG * df_gL[:, L_M,
                                                              np.newaxis,
                                                              np.newaxis],
                                               dv_g, axes=([0, 0]))
                coefatom_GG = np.sum(coefatomang_MGG * coefatomrad_MGG, axis=0)
                KxcPAW_GG[myG_G] += coefatom_GG * coefatomR_GG

            self.world.sum(KxcPAW_GG)
            Kxc_GG += KxcPAW_GG

        return Kxc_GG / vol

    def _ang_int(self, f_nA):
        """ Make surface integral on a sphere using Lebedev quadrature """
        return 4. * np.pi * np.tensordot(weight_n, f_nA, axes=([0], [0]))

    def _initialize_rshe_convcrit(self, df_ng, nL):
        """
        Initialize the convergence criteria for the expansion in real spherical
        harmonics by finding the surface norm square of df as a function of g
        (df is assumed to be a real function).
        The convergence of the expansion is tracked by comparing the surface
        norm square calculated using the expansion and the full result.
        """
        # Calculate surface norm square
        dfSns_g = self._ang_int(df_ng ** 2)
        self.dfSns_g = dfSns_g
        self.dfSns_gL = np.repeat(dfSns_g, nL).reshape(dfSns_g.shape[0], nL)
        # Find PAW correction range
        self.dfmask_g = np.where(self.dfSns_g > 0.)
        ng = np.max(self.dfmask_g) + 1

        # Initialize convergence criteria
        self.rsheconvmin = 0.

        return ng

    def _add_rshe_coefficients(self, df_ng, df_gL, Y_nL, l):
        """
        Adds the l-components in the real spherical harmonic expansion
        of df_ng to df_gL.
        Assumes df_ng to be a real function.
        """
        nm = 2 * l + 1
        L_L = np.arange(l**2, l**2 + nm)
        df_ngm = np.repeat(df_ng, nm, axis=1).reshape((*df_ng.shape, nm))
        Y_ngm = np.repeat(Y_nL[:, L_L],
                          df_ng.shape[1], axis=0).reshape((*df_ng.shape, nm))
        df_gL[:, L_L] = self._ang_int(Y_ngm * df_ngm)

    def _evaluate_rshe_coefficients(self, f_gL):
        """
        Checks weither some (l,m)-coefficients are very small for all g,
        in that case they can be excluded from the expansion.
        """
        fc_L = np.sum(f_gL[self.dfmask_g]**2 / self.dfSns_gL[self.dfmask_g],
                      axis=0)
        self.rsheL_M = fc_L > (1. - self.rshecc) * 1.e-3

    def _evaluate_rshe_convergence(self, f_gL):
        """
        Find the minimal fraction of f_ng captured by the expansion
        in real spherical harmonics f_gL.
        """
        self._evaluate_rshe_coefficients(f_gL)

        rsheconv_g = np.ones(f_gL.shape[0])
        dfSns_g = np.sum(f_gL[:, self.rsheL_M]**2, axis=1)
        rsheconv_g[self.dfmask_g] = dfSns_g[self.dfmask_g]
        rsheconv_g[self.dfmask_g] /= self.dfSns_g[self.dfmask_g]

        self.rsheconvmin = np.min(rsheconv_g)

    def add_fxc(self, gd, n_sg, fxc_g):
        raise NotImplementedError


class AdiabaticDensityKernelCalculator(AdiabaticKernelCalculator):

    def __init__(self, fd, world,
                 rshe=0.99,
                 ecut=None,
                 density_cut=None):
        """
        density_cut : float
            cutoff density below which f_xc is set to zero
        """

        self.density_cut = density_cut

        AdiabaticKernelCalculator.__init__(self, fd, world, rshe, ecut)

        self.permitted_functionals += ['ALDA_x', 'ALDA_X', 'ALDA']

    def __call__(self, pd, calc, functional):

        Kxc_GG = AdiabaticKernelCalculator.__call__(self, pd, calc,
                                                    functional)

        if pd.kd.gamma:
            Kxc_GG[0, :] = 0.0
            Kxc_GG[:, 0] = 0.0

        return Kxc_GG

    def add_fxc(self, gd, n_sG, fxc_G):
        """
        Calculate fxc, using the cutoffs from input above

        ALDA_x is an explicit algebraic version
        ALDA_X uses the libxc package
        """

        _calculate_fxc = self._calculate_fxc
        density_cut = self.density_cut

        # Mask small n
        n_G = np.sum(n_sG, axis=0)
        if density_cut:
            npos_G = np.abs(n_G) > density_cut
        else:
            npos_G = np.full(np.shape(n_G), True, np.array(True).dtype)

        # Calculate fxc
        fxc_G[npos_G] += _calculate_fxc(gd, n_sG)[npos_G]

    def _calculate_fxc(self, gd, n_sG):
        if self.functional == 'ALDA_x':
            n_G = np.sum(n_sG, axis=0)
            fx_G = -1. / 3. * (3. / np.pi)**(1. / 3.) * n_G**(-2. / 3.)
            return fx_G
        else:
            fxc_sG = np.zeros_like(n_sG)
            xc = XC(self.functional[1:])
            xc.calculate_fxc(gd, n_sG, fxc_sG)

            return fxc_sG[0]


class AdiabaticTransverseKernelCalculator(AdiabaticKernelCalculator):

    def __init__(self, fd, world,
                 rshe=0.99,
                 ecut=None,
                 density_cut=None,
                 spinpol_cut=None):
        """
        density_cut : float
            cutoff density below which f_xc is set to zero
        spinpol_cut : float
            cutoff spin polarization. Below, f_xc is evaluated in zeta=0 limit
        """

        self.density_cut = density_cut
        self.spinpol_cut = spinpol_cut

        AdiabaticKernelCalculator.__init__(self, fd, world, rshe, ecut)

        self.permitted_functionals += ['ALDA_x', 'ALDA_X', 'ALDA']

    def add_fxc(self, gd, n_sG, fxc_G):
        """
        Calculate fxc, using the cutoffs from input above

        ALDA_x is an explicit algebraic version
        ALDA_X uses the libxc package
        """

        _calculate_pol_fxc = self._calculate_pol_fxc
        _calculate_unpol_fxc = self._calculate_unpol_fxc
        spinpol_cut = self.spinpol_cut
        density_cut = self.density_cut

        # Mask small zeta
        n_G, m_G = None, None
        if spinpol_cut is not None:
            m_G = n_sG[0] - n_sG[1]
            n_G = n_sG[0] + n_sG[1]
            zetasmall_G = np.abs(m_G / n_G) < spinpol_cut
        else:
            zetasmall_G = np.full(np.shape(n_sG[0]), False,
                                  np.array(False).dtype)

        # Mask small n
        if density_cut:
            if n_G is None:
                n_G = n_sG[0] + n_sG[1]
            npos_G = np.abs(n_G) > density_cut
        else:
            npos_G = np.full(np.shape(n_sG[0]), True, np.array(True).dtype)

        # Don't use small zeta limit if n is small
        zetasmall_G = np.logical_and(zetasmall_G, npos_G)

        # In small zeta limit, use unpolarized fxc
        if zetasmall_G.any():
            if n_G is None:
                n_G = n_sG[0] + n_sG[1]
            fxc_G[zetasmall_G] += _calculate_unpol_fxc(gd, n_G)[zetasmall_G]

        # Set fxc to zero if n is small
        allfine_G = np.logical_and(np.invert(zetasmall_G), npos_G)

        # Above both spinpol_cut and density_cut calculate polarized fxc
        if m_G is None:
            m_G = n_sG[0] - n_sG[1]
        fxc_G[allfine_G] += _calculate_pol_fxc(gd, n_sG, m_G)[allfine_G]

    def _calculate_pol_fxc(self, gd, n_sG, m_G):
        """ Calculate polarized fxc """

        assert np.shape(m_G) == np.shape(n_sG[0])

        if self.functional == 'ALDA_x':
            fx_G = - (6. / np.pi)**(1. / 3.) \
                * (n_sG[0]**(1. / 3.) - n_sG[1]**(1. / 3.)) / m_G
            return fx_G
        else:
            v_sG = np.zeros(np.shape(n_sG))
            xc = XC(self.functional[1:])
            xc.calculate(gd, n_sG, v_sg=v_sG)

            return (v_sG[0] - v_sG[1]) / m_G

    def _calculate_unpol_fxc(self, gd, n_G):
        """ Calculate unpolarized fxc """
        fx_G = - (3. / np.pi)**(1. / 3.) * 2. / 3. * n_G**(-2. / 3.)
        if self.functional in ('ALDA_x', 'ALDA_X'):
            return fx_G
        else:
            # From Perdew & Wang 1992
            A = 0.016887
            a1 = 0.11125
            b1 = 10.357
            b2 = 3.6231
            b3 = 0.88026
            b4 = 0.49671

            rs_G = 3. / (4. * np.pi) * n_G**(-1. / 3.)
            X_G = 2. * A * (b1 * rs_G**(1. / 2.)
                            + b2 * rs_G + b3 * rs_G**(3. / 2.) + b4 * rs_G**2.)
            ac_G = 2. * A * (1 + a1 * rs_G) * np.log(1. + 1. / X_G)

            fc_G = 2. * ac_G / n_G

            return fx_G + fc_G


def calculate_renormalized_kernel(pd, calc, functional, fd):
    """Renormalized kernel"""

    from gpaw.xc.fxc import KernelDens
    kernel = KernelDens(calc,
                        functional,
                        [pd.kd.bzk_kc[0]],
                        fd,
                        calc.wfs.kd.N_c,
                        None,
                        ecut=pd.ecut * Ha,
                        tag='',
                        timer=Timer())

    kernel.calculate_fhxc()
    r = Reader('fhxc_%s_%s_%s_%s.gpw' %
               ('', functional, pd.ecut * Ha, 0))
    Kxc_sGG = np.array([r.get('fhxc_sGsG')])

    v_G = 4 * np.pi / pd.G2_qG[0]
    Kxc_sGG[0] -= np.diagflat(v_G)

    if pd.kd.gamma:
        Kxc_sGG[:, 0, :] = 0.0
        Kxc_sGG[:, :, 0] = 0.0

    return Kxc_sGG


def calculate_lr_kernel(pd, calc, alpha=0.2):
    """Long range kernel: fxc = \alpha / |q+G|^2"""

    assert pd.kd.gamma

    f_G = np.zeros(len(pd.G2_qG[0]))
    f_G[0] = -alpha
    f_G[1:] = -alpha / pd.G2_qG[0][1:]

    return np.array([np.diag(f_G)])


def calculate_dm_kernel(pd, calc):
    """Density matrix kernel"""

    assert pd.kd.gamma

    nv = calc.wfs.setups.nvalence
    psit_nG = np.array([calc.wfs.kpt_u[0].psit_nG[n]
                        for n in range(4 * nv)])
    vol = np.linalg.det(calc.wfs.gd.cell_cv)
    Ng = np.prod(calc.wfs.gd.N_c)
    rho_GG = np.dot(psit_nG.conj().T, psit_nG) * vol / Ng**2

    maxG2 = np.max(pd.G2_qG[0])
    cut_G = np.arange(calc.wfs.pd.ngmax)[calc.wfs.pd.G2_qG[0] <= maxG2]

    G_G = pd.G2_qG[0]**0.5
    G_G[0] = 1.0

    Kxc_GG = np.diagflat(4 * np.pi / G_G**2)
    Kxc_GG = np.dot(Kxc_GG, rho_GG.take(cut_G, 0).take(cut_G, 1))
    Kxc_GG -= 4 * np.pi * np.diagflat(1.0 / G_G**2)

    return np.array([Kxc_GG])


def get_bootstrap_kernel(pd, chi0, chi0_wGG, fd):
    """ Bootstrap kernel (see below) """

    if chi0.world.rank == 0:
        chi0_GG = chi0_wGG[0]
        if chi0.world.size > 1:
            # If size == 1, chi0_GG is not contiguous, and broadcast()
            # will fail in debug mode.  So we skip it until someone
            # takes a closer look.
            chi0.world.broadcast(chi0_GG, 0)
    else:
        nG = pd.ngmax
        chi0_GG = np.zeros((nG, nG), complex)
        chi0.world.broadcast(chi0_GG, 0)

    return calculate_bootstrap_kernel(pd, chi0_GG, fd)


def calculate_bootstrap_kernel(pd, chi0_GG, fd):
    """Bootstrap kernel PRL 107, 186401"""

    if pd.kd.gamma:
        v_G = np.zeros(len(pd.G2_qG[0]))
        v_G[0] = 4 * np.pi
        v_G[1:] = 4 * np.pi / pd.G2_qG[0][1:]
    else:
        v_G = 4 * np.pi / pd.G2_qG[0]

    nG = len(v_G)
    K_GG = np.diag(v_G)

    fxc_GG = np.zeros((nG, nG), dtype=complex)
    dminv_GG = np.zeros((nG, nG), dtype=complex)

    for iscf in range(120):
        dminvold_GG = dminv_GG.copy()
        Kxc_GG = K_GG + fxc_GG

        chi_GG = np.dot(np.linalg.inv(np.eye(nG, nG)
                                      - np.dot(chi0_GG, Kxc_GG)), chi0_GG)
        dminv_GG = np.eye(nG, nG) + np.dot(K_GG, chi_GG)

        alpha = dminv_GG[0, 0] / (K_GG[0, 0] * chi0_GG[0, 0])
        fxc_GG = alpha * K_GG
        print(iscf, 'alpha =', alpha, file=fd)
        error = np.abs(dminvold_GG - dminv_GG).sum()
        if np.sum(error) < 0.1:
            print('Self consistent fxc finished in %d iterations !' % iscf,
                  file=fd)
            break
        if iscf > 100:
            print('Too many fxc scf steps !', file=fd)

    return np.array([fxc_GG])



import numpy as np

from ase.build import molecule
from gpaw import GPAW
from gpaw.lcaotddft import LCAOTDDFT
from gpaw.poisson import PoissonSolver
from gpaw.lcaotddft.dipolemomentwriter import DipoleMomentWriter
from gpaw.mpi import world

from gpaw.test import equal

# Atoms
atoms = molecule('Na2')
atoms.center(vacuum=4.0)

# Ground-state calculation
calc = GPAW(nbands=2, h=0.4, setups=dict(Na='1'),
            basis='dzp', mode='lcao',
            poissonsolver=PoissonSolver(eps=1e-16),
            convergence={'density': 1e-8},
            xc='LDA',
            txt='gs.out')
atoms.set_calculator(calc)
energy = atoms.get_potential_energy()
calc.write('gs.gpw', mode='all')

# Time-propagation calculation without fxc
td_calc = LCAOTDDFT('gs.gpw', txt='td.out')
DipoleMomentWriter(td_calc, 'dm.dat')
td_calc.absorption_kick(np.ones(3) * 1e-5)
td_calc.propagate(20, 4)
world.barrier()

# Check dipole moment file
ref = np.loadtxt('dm.dat').ravel()

# Time-propagation calculation with fxc=xc
td_calc = LCAOTDDFT('gs.gpw', fxc='LDA', txt='td_fxc.out')
DipoleMomentWriter(td_calc, 'dm_fxc.dat')
td_calc.absorption_kick(np.ones(3) * 1e-5)
td_calc.propagate(20, 1)
td_calc.write('td_fxc.gpw', mode='all')

# Restart from the restart point
td_calc = LCAOTDDFT('td_fxc.gpw', txt='td_fxc2.out')
DipoleMomentWriter(td_calc, 'dm_fxc.dat')
td_calc.propagate(20, 3)

# Check dipole moment file
data = np.loadtxt('dm_fxc.dat')[[0, 1, 2, 4, 5, 6]].ravel()

tol = 1e-9
equal(data, ref, tol)



from __future__ import print_function
import numpy as np
from gpaw.utilities.ewald import Ewald

verbose = True

# References for Madelung constants with homogenous background charges:
# M.P. Marder, "Condensed Matter Physics", J. Wiley, (2000),
# scaled to the Wigner-Seitz radius.
# References for Madelung constants of ionic crystals:
# Y. Sakamoto, The J. of Chem. Phys., vol. 28, (1958), p. 164,
# scaled to the nearest neighbor distance.

if 1: #fcc
    cell = np.array([[0., .5, .5],
                      [.5, .0, .5],
                      [.5, .5, .0]])
    basis = np.array([[0.,0.,0.]])
    charges =  np.array([1.])
    r = np.array([0.0,0.0,0.0])
    charges = np.array([1])
    ewald = Ewald(cell)
    e = ewald.get_electrostatic_potential(r, basis, charges, excludefroml0=0)
    a_this = -e * ewald.get_wigner_seitz_radius(sum(charges))
    a_ref = 1.79175
    if verbose:
        print('Madelung energy, fcc:', a_this, a_this-a_ref)
    assert abs(a_this-a_ref) < 1e-5

if 1: #hcp
    a = 1.
    c = np.sqrt(8./3.)*a
    cell =  np.array([[.5*a, -0.5*np.sqrt(3)*a,0],
                      [.5*a, 0.5*np.sqrt(3)*a,0],
                      [0., 0., c]])
    basis = np.array([[.5*a, .5/np.sqrt(3)*a, 0.],
                      [.5*a, -.5/np.sqrt(3)*a, 0.5*c]])
    r = np.array([.5*a, .5/np.sqrt(3)*a, 0.])
    charges =  np.array([1.,1.])
    ewald = Ewald(cell)
    e = ewald.get_electrostatic_potential(r, basis, charges, excludefroml0=0)
    a_this = -e * ewald.get_wigner_seitz_radius(sum(charges))
    a_ref = 1.79168
    if verbose:
        print('Madelung energy, hcp:', a_this, a_this-a_ref)
    assert abs(a_this-a_ref) < 1e-5
    
if 1: #Simple Cubic
    cell = np.diag([1.,1.,1.])
    basis = np.array([[0.,0.,0.]])
    charges =  np.array([1.])
    r = np.array([0.0,0.0,0.0])
    charges = np.array([1])
    ewald = Ewald(cell)
    e = ewald.get_electrostatic_potential(r, basis, charges, excludefroml0=0)
    a_this = -e*ewald.get_wigner_seitz_radius(1)
    a_ref = 1.76012
    if verbose:
        print('Madelung energy, sc:',  a_this, a_this-a_ref)
    assert abs(a_this-a_ref) < 1e-5

if 1: # NaCl
    cell = np.array([[0., .5, .5],
                      [.5, .0, .5],
                      [.5, .5, .0]])
    basis = np.array([[0.,0.,0.],
                      [.5,.5,.5]])
    r = np.array([0.0,0.0,0.0])
    charges =  np.array([1,-1])
    ewald = Ewald(cell)
    e = ewald.get_electrostatic_potential(r, basis, charges, excludefroml0=0)
    a_this = - 0.5 * e
    a_ref = 1.7475645946331822
    if verbose:
        print('Madelung constant, NaCl (B1):', a_this, a_this-a_ref)
    assert abs(a_this-a_ref) < 1e-13


if 0: # NaCl
    cell =np.diag((1.,1.,1.))
    basis = np.array([[0.,0.,0.],
                      [.5,.5,.0],
                      [.5,.0,.5],
                      [.0,.5,.5],
                      [.5,0.,0.],
                      [.0,.5,.0],
                      [.0,.0,.5],
                      [.5,.5,.5]]
                     )
    r = np.array([0.0,0.0,0.0])
    charges = np.array([1,1,1,1,-1,-1,-1,-1])
    ewald = Ewald(cell)
    e = ewald.get_electrostatic_potential(r, basis, charges, excludefroml0=0)
    a_this = - 0.5 * e
    a_ref = 1.7475645946331822
    if verbose:
        print('Madelung constant, NaCl (B1):', a_this, a_this-a_ref)
    assert abs(a_this-a_ref) < 1e-13

if 1: # CsCl
    cell = np.diag((1.,1.,1.))
    basis = np.array([[0.,0.,0.],
                      [.5, .5, .5]])
    r = np.array([0.0,0.0,0.0])
    charges =  np.array([1,-1])
    ewald = Ewald(cell)
    e = ewald.get_electrostatic_potential(r, basis, charges, excludefroml0=0)
    a_this = - .5 * np.sqrt(3.) * e
    a_ref = 1.76267477307099
    if verbose:
        print('Madelung constant, CsCl (B2):', a_this, a_this-a_ref)
    assert abs(a_this-a_ref) < 1e-13
    
if 1: # Zincblende, cubic ZnS
    cell =  np.array([[.0,.5,.5],
                      [.5,.0,.5],
                      [.5,.5,.0]])
    basis = np.array([[0.,0.,0.],
                      [.25,.25,.25]])
    r = np.array([0.0,0.0,0.0])
    charges =  np.array([1,-1])
    ewald = Ewald(cell)
    e = ewald.get_electrostatic_potential(r, basis, charges, excludefroml0=0)
    a_this = - 0.25 * np.sqrt(3) * e
    a_ref = 1.63805505338879
    if verbose:
        print('Madelung constant, Zincblende (B3):', a_this, a_this-a_ref)
    assert abs(a_this-a_ref) < 1e-13

if 1: # Ideal Wurtzite, ZnS, (B4)
    a = 1.
    c = np.sqrt(8./3.)*a
    u = 3./8.
    cell =  np.array([[.5*a, -0.5*np.sqrt(3)*a,0],
                      [.5*a, 0.5*np.sqrt(3)*a,0],
                      [0., 0., c]])
    basis = np.array([[.5*a, .5/np.sqrt(3)*a, 0.],
                       [.5*a, -.5/np.sqrt(3)*a, 0.5*c],
                       [.5*a, .5/np.sqrt(3)*a, u*c],
                       [.5*a, -.5/np.sqrt(3)*a, (.5+u)*c]])
    r = np.array([.5*a, .5/np.sqrt(3)*a, 0.])
    charges =  np.array([1.,1.,-1,-1])
    ewald = Ewald(cell)
    e = ewald.get_electrostatic_potential(r, basis, charges, excludefroml0=0)
    a_this = - 1. * u * c * e
    a_ref = 1.64132162737
    if verbose:
        print('Madelung constant, Ideal Wurtzite (B4):', a_this, a_this-a_ref)
    assert abs(a_this-a_ref) < 1e-11



# http://listserv.fysik.dtu.dk/pipermail/gpaw-developers/2014-February/004374.html
from __future__ import print_function
from gpaw import GPAW, PoissonSolver
from gpaw.test import equal
from ase.build import molecule


for i in range(12):
    mol = molecule('H2')
    mol.center(vacuum=1.5)
    calc = GPAW(h=0.3, nbands=2, mode='lcao', txt=None, basis='sz(dzp)',
                poissonsolver=PoissonSolver(eps=17.), xc='oldLDA')

    def stop():
        calc.scf.converged = True

    calc.attach(stop, 1)
    mol.set_calculator(calc)
    e = mol.get_potential_energy()
    if i == 0:
        eref = e
    if calc.wfs.world.rank == 0:
        print(repr(e))
    equal(e - eref, 0, 1.e-12)



from __future__ import print_function

import numpy as np

from ase.build import molecule
from ase.parallel import parprint

from gpaw import GPAW
from gpaw.analyse.multipole import Multipole
from gpaw.cluster import Cluster
from gpaw.test import equal

h = 0.3

s = Cluster(molecule('H2O'))
s.minimal_box(3., h)

gpwname = 'H2O_h' + str(h) + '.gpw'
try:
    # XXX check why this fails in parallel
    calc = GPAW(gpwname + 'failsinparallel', txt=None)
    atoms = calc.get_atoms()
    calc.density.set_positions(atoms.get_scaled_positions() % 1.0)
    calc.density.interpolate_pseudo_density()
    calc.density.calculate_pseudo_charge()
except IOError:
    calc = GPAW(h=h, charge=0, txt=None)
    calc.calculate(s)
    calc.write(gpwname)

dipole_c = calc.get_dipole_moment()
parprint('Dipole', dipole_c) 

center = np.array([1,1,1]) * 50.
mp = Multipole(center, calc, lmax=2)
q_L = mp.expand(-calc.density.rhot_g)
parprint('Multipole', q_L)

# The dipole moment is independent of the center
equal(dipole_c[2], q_L[2], 1e-10)

mp.to_file(calc, mode='w')



from __future__ import print_function, division

import functools
import os
from time import ctime

import numpy as np
from ase.units import Hartree
from ase.utils import convert_string_to_fd
from ase.utils.timing import timer, Timer
from scipy.special.orthogonal import p_roots

import gpaw.mpi as mpi
from gpaw import GPAW
from gpaw.kpt_descriptor import KPointDescriptor
from gpaw.response.chi0 import Chi0
from gpaw.response.kernels import get_coulomb_kernel
from gpaw.response.wstc import WignerSeitzTruncatedCoulomb
from gpaw.wavefunctions.pw import PWDescriptor, count_reciprocal_vectors


def rpa(filename, ecut=200.0, blocks=1, extrapolate=4):
    """Calculate RPA energy.

    filename: str
        Name of restart-file.
    ecut: float
        Plane-wave cutoff.
    blocks: int
        Split polarizability matrix in this many blocks.
    extrapolate: int
        Number of cutoff energies to use for extrapolation.
    """
    name, ext = filename.rsplit('.', 1)
    assert ext == 'gpw'
    from gpaw.xc.rpa import RPACorrelation
    rpa = RPACorrelation(name, name + '-rpa.dat',
                         nblocks=blocks,
                         txt=name + '-rpa.txt')
    rpa.calculate(ecut=ecut * (1 + 0.5 * np.arange(extrapolate))**(-2 / 3))


class RPACorrelation:
    def __init__(self, calc, xc='RPA', filename=None,
                 skip_gamma=False, qsym=True, nlambda=None,
                 nfrequencies=16, frequency_max=800.0, frequency_scale=2.0,
                 frequencies=None, weights=None, truncation=None,
                 world=mpi.world, nblocks=1, txt='-'):
        """Creates the RPACorrelation object

        calc: str or calculator object
            The string should refer to the .gpw file contaning KS orbitals
        xc: str
            Exchange-correlation kernel. This is only different from RPA when
            this object is constructed from a different module - e.g. fxc.py
        filename: str
            txt output
        skip_gamme: bool
            If True, skip q = [0,0,0] from the calculation
        qsym: bool
            Use symmetry to reduce q-points
        nlambda: int
            Number of lambda points. Only used for numerical coupling
            constant integration involved when called from fxc.py
        nfrequencies: int
            Number of frequency points used in the Gauss-Legendre integration
        frequency_max: float
            Largest frequency point in Gauss-Legendre integration
        frequency_scale: float
            Determines density of frequency points at low frequencies. A slight
            increase to e.g. 2.5 or 3.0 improves convergence wth respect to
            frequency points for metals
        frequencies: list
            List of frequancies for user-specified frequency integration
        weights: list
            list of weights (integration measure) for a user specified
            frequency grid. Must be specified and have the same length as
            frequencies if frequencies is not None
        truncation: str
            Coulomb truncation scheme. Can be either wigner-seitz,
            2D, 1D, or 0D
        world: communicator
        nblocks: int
            Number of parallelization blocks. Frequency parallelization
            can be specified by setting nblocks=nfrequencies and is useful
            for memory consuming calculations
        txt: str
            txt file for saving and loading contributions to the correlation
            energy from different q-points
        """

        if isinstance(calc, str):
            calc = GPAW(calc, txt=None, communicator=mpi.serial_comm)
        self.calc = calc

        self.fd = convert_string_to_fd(txt, world)

        self.timer = Timer()

        if frequencies is None:
            frequencies, weights = get_gauss_legendre_points(nfrequencies,
                                                             frequency_max,
                                                             frequency_scale)
            user_spec = False
        else:
            assert weights is not None
            user_spec = True

        self.omega_w = frequencies / Hartree
        self.weight_w = weights / Hartree

        if nblocks > 1:
            assert len(self.omega_w) % nblocks == 0

        self.nblocks = nblocks
        self.world = world

        self.truncation = truncation
        self.skip_gamma = skip_gamma
        self.ibzq_qc = None
        self.weight_q = None
        self.initialize_q_points(qsym)

        # Energies for all q-vetors and cutoff energies:
        self.energy_qi = []

        self.filename = filename

        self.print_initialization(xc, frequency_scale, nlambda, user_spec)

    def initialize_q_points(self, qsym):
        kd = self.calc.wfs.kd
        self.bzq_qc = kd.get_bz_q_points(first=True)

        if not qsym:
            self.ibzq_qc = self.bzq_qc
            self.weight_q = np.ones(len(self.bzq_qc)) / len(self.bzq_qc)
        else:
            U_scc = kd.symmetry.op_scc
            self.ibzq_qc = kd.get_ibz_q_points(self.bzq_qc, U_scc)[0]
            self.weight_q = kd.q_weights

    def read(self):
        lines = open(self.filename).readlines()[1:]
        n = 0
        self.energy_qi = []
        nq = len(lines) // len(self.ecut_i)
        for q_c in self.ibzq_qc[:nq]:
            self.energy_qi.append([])
            for ecut in self.ecut_i:
                q1, q2, q3, ec, energy = [float(x)
                                          for x in lines[n].split()]
                self.energy_qi[-1].append(energy / Hartree)
                n += 1

                if (abs(q_c - (q1, q2, q3)).max() > 1e-4 or
                    abs(int(ecut * Hartree) - ec) > 0):
                    self.energy_qi = []
                    return

        print('Read %d q-points from file: %s' % (nq, self.filename),
              file=self.fd)
        print(file=self.fd)

    def write(self):
        if self.world.rank == 0 and self.filename:
            fd = open(self.filename, 'w')
            print('#%9s %10s %10s %8s %12s' %
                  ('q1', 'q2', 'q3', 'E_cut', 'E_c(q)'), file=fd)
            for energy_i, q_c in zip(self.energy_qi, self.ibzq_qc):
                for energy, ecut in zip(energy_i, self.ecut_i):
                    print('%10.4f %10.4f %10.4f %8d   %r' %
                          (tuple(q_c) + (ecut * Hartree, energy * Hartree)),
                          file=fd)

    def calculate(self, ecut, nbands=None, spin=False):
        """Calculate RPA correlation energy for one or several cutoffs.

        ecut: float or list of floats
            Plane-wave cutoff(s).
        nbands: int
            Number of bands (defaults to number of plane-waves).
        spin: bool
            Separate spin in response function.
            (Only needed for beyond RPA methods that inherit this function).
        """

        p = functools.partial(print, file=self.fd)

        if isinstance(ecut, (float, int)):
            ecut = ecut * (1 + 0.5 * np.arange(6))**(-2 / 3)
        self.ecut_i = np.asarray(np.sort(ecut)) / Hartree
        ecutmax = max(self.ecut_i)

        if nbands is None:
            p('Response function bands : Equal to number of plane waves')
        else:
            p('Response function bands : %s' % nbands)
        p('Plane wave cutoffs (eV) :', end='')
        for e in self.ecut_i:
            p(' {0:.3f}'.format(e * Hartree), end='')
        p()
        if self.truncation is not None:
            p('Using %s Coulomb truncation' % self.truncation)
        p()

        if self.filename and os.path.isfile(self.filename):
            self.read()
            self.world.barrier()

        chi0 = Chi0(self.calc, frequencies=1j * Hartree * self.omega_w,
                    eta=0.0, intraband=False, hilbert=False,
                    txt='chi0.txt', timer=self.timer, world=self.world,
                    nblocks=self.nblocks)

        self.blockcomm = chi0.blockcomm

        wfs = self.calc.wfs

        if self.truncation == 'wigner-seitz':
            self.wstc = WignerSeitzTruncatedCoulomb(wfs.gd.cell_cv,
                                                    wfs.kd.N_c, self.fd)
        else:
            self.wstc = None

        nq = len(self.energy_qi)
        nw = len(self.omega_w)
        nGmax = max(count_reciprocal_vectors(ecutmax, wfs.gd, q_c)
                    for q_c in self.ibzq_qc[nq:])
        mynGmax = (nGmax + self.nblocks - 1) // self.nblocks

        nx = (1 + spin) * nw * mynGmax * nGmax
        A1_x = np.empty(nx, complex)
        if self.nblocks > 1:
            A2_x = np.empty(nx, complex)
        else:
            A2_x = None

        self.timer.start('RPA')

        for q_c in self.ibzq_qc[nq:]:
            if np.allclose(q_c, 0.0) and self.skip_gamma:
                self.energy_qi.append(len(self.ecut_i) * [0.0])
                self.write()
                p('Not calculating E_c(q) at Gamma')
                p()
                continue

            thisqd = KPointDescriptor([q_c])
            pd = PWDescriptor(ecutmax, wfs.gd, complex, thisqd)
            nG = pd.ngmax
            mynG = (nG + self.nblocks - 1) // self.nblocks
            chi0.Ga = self.blockcomm.rank * mynG
            chi0.Gb = min(chi0.Ga + mynG, nG)

            shape = (1 + spin, nw, chi0.Gb - chi0.Ga, nG)
            chi0_swGG = A1_x[:np.prod(shape)].reshape(shape)
            chi0_swGG[:] = 0.0

            if np.allclose(q_c, 0.0):
                chi0_swxvG = np.zeros((1 + spin, nw, 2, 3, nG), complex)
                chi0_swvv = np.zeros((1 + spin, nw, 3, 3), complex)
            else:
                chi0_swxvG = None
                chi0_swvv = None

            # First not completely filled band:
            m1 = chi0.nocc1
            p('# %s  -  %s' % (len(self.energy_qi), ctime().split()[-2]))
            p('q = [%1.3f %1.3f %1.3f]' % tuple(q_c))

            energy_i = []
            for ecut in self.ecut_i:
                if ecut == ecutmax:
                    # Nothing to cut away:
                    cut_G = None
                    m2 = nbands or nG
                else:
                    cut_G = np.arange(nG)[pd.G2_qG[0] <= 2 * ecut]
                    m2 = len(cut_G)

                p('E_cut = %d eV / Bands = %d:' % (ecut * Hartree, m2))
                self.fd.flush()

                energy = self.calculate_q(chi0, pd,
                                          chi0_swGG, chi0_swxvG, chi0_swvv,
                                          m1, m2, cut_G, A2_x)

                energy_i.append(energy)
                m1 = m2

                a = 1 / chi0.kncomm.size
                if ecut < ecutmax and a != 1.0:
                    # Chi0 will be summed again over chicomm, so we divide
                    # by its size:
                    chi0_swGG *= a
                    #if chi0_swxvG is not None:
                    #    chi0_swxvG *= a
                    #    chi0_swvv *= a

            self.energy_qi.append(energy_i)
            self.write()
            p()

        e_i = np.dot(self.weight_q, np.array(self.energy_qi))
        p('==========================================================')
        p()
        p('Total correlation energy:')
        for e_cut, e in zip(self.ecut_i, e_i):
            p('%6.0f:   %6.4f eV' % (e_cut * Hartree, e * Hartree))
        p()

        self.energy_qi = []  # important if another calculation is performed

        if len(e_i) > 1:
            self.extrapolate(e_i)

        p('Calculation completed at: ', ctime())
        p()

        self.timer.stop('RPA')
        self.timer.write(self.fd)
        self.fd.flush()

        return e_i * Hartree

    @timer('chi0(q)')
    def calculate_q(self, chi0, pd, chi0_swGG, chi0_swxvG, chi0_swvv,
                    m1, m2, cut_G, A2_x):
        chi0_wGG = chi0_swGG[0]
        if chi0_swxvG is not None:
            chi0_wxvG = chi0_swxvG[0]
            chi0_wvv = chi0_swvv[0]
        else:
            chi0_wxvG = None
            chi0_wvv = None
        chi0._calculate(pd, chi0_wGG, chi0_wxvG, chi0_wvv,
                        m1, m2, spins='all', extend_head=False)

        print('E_c(q) = ', end='', file=self.fd)

        chi0_wGG = chi0.redistribute(chi0_wGG, A2_x)

        if not pd.kd.gamma:
            e = self.calculate_energy(pd, chi0_wGG, cut_G)
            print('%.3f eV' % (e * Hartree), file=self.fd)
            self.fd.flush()
        else:
            from ase.dft import monkhorst_pack
            kd = self.calc.wfs.kd
            N = 4
            N_c = np.array([N, N, N])
            if self.truncation is not None:
                N_c[kd.N_c == 1] = 1
            q_qc = monkhorst_pack(N_c) / kd.N_c
            q_qc *= 1.0e-6
            U_scc = kd.symmetry.op_scc
            q_qc = kd.get_ibz_q_points(q_qc, U_scc)[0]
            weight_q = kd.q_weights
            q_qv = 2 * np.pi * np.dot(q_qc, pd.gd.icell_cv)

            nw = len(self.omega_w)
            mynw = nw // self.nblocks
            w1 = self.blockcomm.rank * mynw
            w2 = w1 + mynw
            a_qw = np.sum(np.dot(chi0_wvv[w1:w2], q_qv.T) * q_qv.T, axis=1).T
            a0_qwG = np.dot(q_qv, chi0_wxvG[w1:w2, 0])
            a1_qwG = np.dot(q_qv, chi0_wxvG[w1:w2, 1])

            e = 0
            for iq in range(len(q_qv)):
                chi0_wGG[:, 0] = a0_qwG[iq]
                chi0_wGG[:, :, 0] = a1_qwG[iq]
                chi0_wGG[:, 0, 0] = a_qw[iq]
                ev = self.calculate_energy(pd, chi0_wGG, cut_G,
                                           q_v=q_qv[iq])
                e += ev * weight_q[iq]
            print('%.3f eV' % (e * Hartree), file=self.fd)
            self.fd.flush()

        return e

    @timer('Energy')
    def calculate_energy(self, pd, chi0_wGG, cut_G, q_v=None):
        """Evaluate correlation energy from chi0."""

        sqrV_G = get_coulomb_kernel(pd, self.calc.wfs.kd.N_c, q_v=q_v,
                                    truncation=self.truncation,
                                    wstc=self.wstc)**0.5
        if cut_G is not None:
            sqrV_G = sqrV_G[cut_G]
        nG = len(sqrV_G)

        e_w = []
        for chi0_GG in chi0_wGG:
            if cut_G is not None:
                chi0_GG = chi0_GG.take(cut_G, 0).take(cut_G, 1)

            e_GG = np.eye(nG) - chi0_GG * sqrV_G * sqrV_G[:, np.newaxis]
            e = np.log(np.linalg.det(e_GG)) + nG - np.trace(e_GG)
            e_w.append(e.real)

        E_w = np.zeros_like(self.omega_w)
        self.blockcomm.all_gather(np.array(e_w), E_w)
        energy = np.dot(E_w, self.weight_w) / (2 * np.pi)
        self.E_w = E_w
        return energy

    def extrapolate(self, e_i):
        print('Extrapolated energies:', file=self.fd)
        ex_i = []
        for i in range(len(e_i) - 1):
            e1, e2 = e_i[i:i + 2]
            x1, x2 = self.ecut_i[i:i + 2]**-1.5
            ex = (e1 * x2 - e2 * x1) / (x2 - x1)
            ex_i.append(ex)

            print('  %4.0f -%4.0f:  %5.3f eV' % (self.ecut_i[i] * Hartree,
                                                 self.ecut_i[i + 1] * Hartree,
                                                 ex * Hartree),
                  file=self.fd)
        print(file=self.fd)
        self.fd.flush()

        return e_i * Hartree

    def print_initialization(self, xc, frequency_scale, nlambda, user_spec):
        p = functools.partial(print, file=self.fd)
        p('----------------------------------------------------------')
        p('Non-self-consistent %s correlation energy' % xc)
        p('----------------------------------------------------------')
        p('Started at:  ', ctime())
        p()
        p('Atoms                          :',
          self.calc.atoms.get_chemical_formula(mode='hill'))
        p('Ground state XC functional     :', self.calc.hamiltonian.xc.name)
        p('Valence electrons              :', self.calc.wfs.setups.nvalence)
        p('Number of bands                :', self.calc.wfs.bd.nbands)
        p('Number of spins                :', self.calc.wfs.nspins)
        p('Number of k-points             :', len(self.calc.wfs.kd.bzk_kc))
        p('Number of irreducible k-points :', len(self.calc.wfs.kd.ibzk_kc))
        p('Number of q-points             :', len(self.bzq_qc))
        p('Number of irreducible q-points :', len(self.ibzq_qc))
        p()
        for q, weight in zip(self.ibzq_qc, self.weight_q):
            p('    q: [%1.4f %1.4f %1.4f] - weight: %1.3f' %
              (q[0], q[1], q[2], weight))
        p()
        p('----------------------------------------------------------')
        p('----------------------------------------------------------')
        p()
        if nlambda is None:
            p('Analytical coupling constant integration')
        else:
            p('Numerical coupling constant integration using', nlambda,
              'Gauss-Legendre points')
        p()
        p('Frequencies')
        if not user_spec:
            p('    Gauss-Legendre integration with %s frequency points' %
              len(self.omega_w))
            p('    Transformed from [0,oo] to [0,1] using e^[-aw^(1/B)]')
            p('    Highest frequency point at %5.1f eV and B=%1.1f' %
              (self.omega_w[-1] * Hartree, frequency_scale))
        else:
            p('    User specified frequency integration with',
              len(self.omega_w), 'frequency points')
        p()
        p('Parallelization')
        p('    Total number of CPUs          : % s' % self.world.size)
        p('    G-vector decomposition        : % s' % self.nblocks)
        p('    K-point/band decomposition    : % s' %
          (self.world.size // self.nblocks))
        p()


def get_gauss_legendre_points(nw=16, frequency_max=800.0, frequency_scale=2.0):
    y_w, weights_w = p_roots(nw)
    y_w = y_w.real
    ys = 0.5 - 0.5 * y_w
    ys = ys[::-1]
    w = (-np.log(1 - ys))**frequency_scale
    w *= frequency_max / w[-1]
    alpha = (-np.log(1 - ys[-1]))**frequency_scale / frequency_max
    transform = (-np.log(1 - ys))**(frequency_scale - 1) \
        / (1 - ys) * frequency_scale / alpha
    return w, weights_w * transform / 2


class CLICommand:
    """Run RPA-correlation calculation."""

    @staticmethod
    def add_arguments(parser):
        add = parser.add_argument
        add('gpw', metavar='gpw-file')
        add('-e', '--cut-off', type=float, default=100, metavar='ECUT',
            help='Plane-wave cut off energy (eV) for polarization function.')
        add('-b', '--blocks', type=int, default=1, metavar='N',
            help='Split polarization matrix in N blocks.')

    @staticmethod
    def run(args):
        assert args.gpw.endswith('.gpw')
        rpa = RPACorrelation(args.gpw,
                             txt=args.gpw[:-3] + 'rpa.txt',
                             nblocks=args.blocks)
        rpa.calculate([args.cut_off])



from math import pi

import numpy as np
try:
    from scipy.special import spherical_jn

    def sphj(n, z):
        return spherical_jn(range(n), z)

except ImportError:
    from scipy.special import sph_jn

    def sphj(n, z):
        return sph_jn(n - 1, z)[0]


from gpaw.gaunt import nabla, gaunt
from gpaw.spherical_harmonics import Y


def two_phi_planewave_integrals(k_Gv, setup=None, Gstart=0, Gend=None,
                                rgd=None, phi_jg=None,
                                phit_jg=None, l_j=None):
    """Calculate PAW-correction matrix elements with planewaves.

    ::

      /  _       _   ik.r     _     ~   _   ik.r ~   _
      | dr [phi (r) e    phi (r) - phi (r) e    phi (r)]
      /        1            2         1            2

                     ll   -  /     2                      ~       ~
      = 4pi \sum_lm i  Y (k) | dr r  [ phi (r) phi (r) - phi (r) phi (r) j (kr)
                        lm   /            1       2         1       2     ll

          /
        * | d\Omega Y     Y     Y
          /          l1m1  l2m2  lm

    """

    if Gend is None:
        Gend = len(k_Gv)

    if setup is not None:
        rgd = setup.rgd
        l_j = setup.l_j
        # Obtain the phi_j and phit_j
        phi_jg = []
        phit_jg = []
        rcut2 = 2 * max(setup.rcut_j)
        gcut2 = rgd.ceil(rcut2)
        for phi_g, phit_g in zip(setup.data.phi_jg, setup.data.phit_jg):
            phi_g = phi_g.copy()
            phit_g = phit_g.copy()
            phi_g[gcut2:] = phit_g[gcut2:] = 0.
            phi_jg.append(phi_g)
            phit_jg.append(phit_g)
    else:
        assert rgd is not None
        assert phi_jg is not None
        assert l_j is not None

    ng = rgd.N
    r_g = rgd.r_g
    dr_g = rgd.dr_g

    # Construct L (l**2 + m) and j (nl) index
    L_i = []
    j_i = []
    for j, l in enumerate(l_j):
        for m in range(2 * l + 1):
            L_i.append(l**2 + m)
            j_i.append(j)
    ni = len(L_i)
    nj = len(l_j)
    lmax = max(l_j) * 2 + 1

    if setup is not None:
        assert ni == setup.ni and nj == setup.nj

    # Initialize
    npw = k_Gv.shape[0]
    R_jj = np.zeros((nj, nj))
    R_ii = np.zeros((ni, ni))
    phi_Gii = np.zeros((npw, ni, ni), dtype=complex)
    j_lg = np.zeros((lmax, ng))

    # Store (phi_j1 * phi_j2 - phit_j1 * phit_j2 ) for further use
    tmp_jjg = np.zeros((nj, nj, ng))
    for j1 in range(nj):
        for j2 in range(nj):
            tmp_jjg[j1, j2] = (phi_jg[j1] * phi_jg[j2] -
                               phit_jg[j1] * phit_jg[j2])

    G_LLL = gaunt(max(l_j))

    # Loop over G vectors
    for iG in range(Gstart, Gend):
        kk = k_Gv[iG]
        k = np.sqrt(np.dot(kk, kk))  # calculate length of q+G
        
        # Calculating spherical bessel function
        for g, r in enumerate(r_g):
            j_lg[:, g] = sphj(lmax, k * r)

        for li in range(lmax):
            # Radial part
            for j1 in range(nj):
                for j2 in range(nj):
                    R_jj[j1, j2] = np.dot(r_g**2 * dr_g,
                                          tmp_jjg[j1, j2] * j_lg[li])

            for mi in range(2 * li + 1):
                # Angular part
                for i1 in range(ni):
                    L1 = L_i[i1]
                    j1 = j_i[i1]
                    for i2 in range(ni):
                        L2 = L_i[i2]
                        j2 = j_i[i2]
                        R_ii[i1, i2] = G_LLL[L1, L2, li**2 + mi] * R_jj[j1, j2]
                
                if np.allclose(k, 0.):  # Avoid division by zero
                    k = 1.
                phi_Gii[iG] += R_ii * Y(li**2 + mi,
                                        kk[0] / k,
                                        kk[1] / k,
                                        kk[2] / k) * (-1j)**li

    phi_Gii *= 4 * pi

    return phi_Gii.reshape(npw, ni * ni)


def two_phi_nabla_planewave_integrals(k_Gv, setup=None, Gstart=0, Gend=None,
                                      rgd=None, phi_jg=None,
                                      phit_jg=None, l_j=None):
    """Calculate PAW-correction matrix elements with planewaves and gradient.

    ::

      /  _       _   ik.r d       _     ~   _   ik.r d   ~   _
      | dr [phi (r) e     -- phi (r) - phi (r) e     -- phi (r)]
      /        1          dx    2         1          dx    2

    and similar for y and z."""

    if Gend is None:
        Gend = len(k_Gv)

    if setup is not None:
        rgd = setup.rgd
        l_j = setup.l_j
        # Obtain the phi_j and phit_j
        phi_jg = []
        phit_jg = []
        rcut2 = 2 * max(setup.rcut_j)
        gcut2 = rgd.ceil(rcut2)
        for phi_g, phit_g in zip(setup.data.phi_jg, setup.data.phit_jg):
            phi_g = phi_g.copy()
            phit_g = phit_g.copy()
            phi_g[gcut2:] = phit_g[gcut2:] = 0.
            phi_jg.append(phi_g)
            phit_jg.append(phit_g)
    else:
        assert rgd is not None
        assert phi_jg is not None
        assert l_j is not None

    ng = rgd.N
    r_g = rgd.r_g
    dr_g = rgd.dr_g

    # Construct L (l**2 + m) and j (nl) index
    L_i = []
    j_i = []
    for j, l in enumerate(l_j):
        for m in range(2 * l + 1):
            L_i.append(l**2 + m)
            j_i.append(j)
    ni = len(L_i)
    nj = len(l_j)
    lmax = max(l_j) * 2 + 1

    ljdef = 3
    l2max = max(l_j) * (max(l_j) > ljdef) + ljdef * (max(l_j) <= ljdef)
    G_LLL = gaunt(l2max)
    Y_LLv = nabla(2 * l2max)

    if setup is not None:
        assert ni == setup.ni and nj == setup.nj

    # Initialize
    npw = k_Gv.shape[0]
    R1_jj = np.zeros((nj, nj))
    R2_jj = np.zeros((nj, nj))
    R_vii = np.zeros((3, ni, ni))
    phi_vGii = np.zeros((3, npw, ni, ni), dtype=complex)
    j_lg = np.zeros((lmax, ng))

    # Store (phi_j1 * dphidr_j2 - phit_j1 * dphitdr_j2) for further use
    tmp_jjg = np.zeros((nj, nj, ng))
    tmpder_jjg = np.zeros((nj, nj, ng))
    for j1 in range(nj):
        for j2 in range(nj):
            dphidr_g = np.empty_like(phi_jg[j2])
            rgd.derivative(phi_jg[j2], dphidr_g)
            dphitdr_g = np.empty_like(phit_jg[j2])
            rgd.derivative(phit_jg[j2], dphitdr_g)

            tmpder_jjg[j1, j2] = (phi_jg[j1] * dphidr_g -
                                  phit_jg[j1] * dphitdr_g)
            tmp_jjg[j1, j2] = (phi_jg[j1] * phi_jg[j2] -
                               phit_jg[j1] * phit_jg[j2])

    # Loop over G vectors
    for iG in range(Gstart, Gend):
        kk = k_Gv[iG]
        k = np.sqrt(np.dot(kk, kk))  # calculate length of q+G

        # Calculating spherical bessel function
        for g, r in enumerate(r_g):
            j_lg[:, g] = sphj(lmax, k * r)

        for li in range(lmax):
            # Radial part
            for j1 in range(nj):
                for j2 in range(nj):
                    R1_jj[j1, j2] = np.dot(r_g**2 * dr_g,
                                           tmpder_jjg[j1, j2] * j_lg[li])
                    R2_jj[j1, j2] = np.dot(r_g * dr_g,
                                           tmp_jjg[j1, j2] * j_lg[li])

            for mi in range(2 * li + 1):
                if k == 0:
                    Ytmp = Y(li**2 + mi, 1.0, 0, 0)
                else:
                    # Note the spherical bessel gives
                    # zero when k == 0 for li != 0
                    Ytmp = Y(li**2 + mi, kk[0] / k, kk[1] / k, kk[2] / k)

                for v in range(3):
                    Lv = 1 + (v + 2) % 3
                    # Angular part
                    for i1 in range(ni):
                        L1 = L_i[i1]
                        j1 = j_i[i1]
                        for i2 in range(ni):
                            L2 = L_i[i2]
                            j2 = j_i[i2]
                            l2 = l_j[j2]

                            R_vii[v, i1, i2] = (
                                (4 * pi / 3)**0.5 *
                                np.dot(G_LLL[L1, L2],
                                       G_LLL[Lv, li**2 + mi]) *
                                (R1_jj[j1, j2] - l2 *
                                 R2_jj[j1, j2]))

                            R_vii[v, i1, i2] += (R2_jj[j1, j2] *
                                                 np.dot(G_LLL[L1, li**2 + mi],
                                                        Y_LLv[:, L2, v]))

                phi_vGii[:, iG] += (R_vii * Ytmp * (-1j)**li)

    phi_vGii *= 4 * pi

    return phi_vGii.reshape(3, npw, ni * ni)



import numpy as np
from ase import Atoms

import gpaw.dscf as dscf
from gpaw import GPAW, MixerSum
from gpaw.mpi import rank

L = 10.0

eigensolver = 'cg'
xc = 'PBE'

E0 = []
E1 = []
d0 = 1.4
delta = 0.05
for i in range(3):
    d = d0 + (i - 1) * delta
    atoms = Atoms('N2',
                  [[0.5 * L, 0.5 * L, 0.5 * L - 0.5 * d],
                   [0.5 * L, 0.5 * L, 0.5 * L + 0.5 * d]])
    atoms.set_cell([L, L, L])

    calc_gs = GPAW(h=0.2,
                   nbands=-5,
                   xc=xc,
                   width=0.05,
                   eigensolver=eigensolver,
                   spinpol=True,
                   txt=None,
                   mixer=MixerSum(beta=0.1, nmaxold=5, weight=50.0),
                   convergence={'energy': 100,
                                'density': 100,
                                'eigenstates': 1.0e-9,
                                'bands': -1})
    atoms.set_calculator(calc_gs)

    E0.append(atoms.get_potential_energy())
    if i == 1:
        F0 = atoms.get_forces()

    calc_es = GPAW(h=0.2,
                   nbands=-5,
                   xc=xc,
                   width=0.05,
                   eigensolver=eigensolver,
                   spinpol=True,
                   txt=None,
                   mixer=MixerSum(beta=0.1, nmaxold=5, weight=50.0),
                   convergence={'energy': 100,
                                'density': 100,
                                'eigenstates': 1.0e-9,
                                'bands': -1})

    n = 5  # LUMO
    molecule = [0, 1]
    wf_u = [kpt.psit_nG[n] for kpt in calc_gs.wfs.kpt_u]
    p_uai = [dict([(molecule[a], P_ni[n]) for a, P_ni in kpt.P_ani.items()])
             for kpt in calc_gs.wfs.kpt_u]

    atoms.set_calculator(calc_es)
    lumo = dscf.AEOrbital(calc_es, wf_u, p_uai)
    dscf.dscf_calculation(calc_es, [[1.0, lumo, 1]], atoms)

    E1.append(atoms.get_potential_energy())
    if i == 1:
        F1 = atoms.get_forces()

f0 = np.sqrt(((F0[1, :] - F0[0, :])**2).sum()) * 0.5
f0b = (E0[1] - E0[0]) / delta     # backward gradient
f0f = (E0[2] - E0[1]) / delta     # forward gradient
if rank == 0:
    print('Ground state')
    print(E0)
    print(f0b, '<', f0, '<', f0f)
assert f0 > f0b
assert f0 < f0f

f1 = np.sqrt(((F1[1, :] - F1[0, :])**2).sum()) * 0.5
f1b = (E1[1] - E1[0]) / delta     # backward gradient
f1f = (E1[2] - E1[1]) / delta     # forward gradient
if rank == 0:
    print('Excited state')
    print(E1)
    print(f1b, '<', f1, '<', f1f)
assert f1 > f1b
assert f1 < f1f



import os
import sys

from ase.cli.completion import update, CLICommand

from gpaw.cli.main import commands

my_dir, _ = os.path.split(os.path.realpath(__file__))
script = os.path.join(my_dir, 'complete.py')

CLICommand.cmd = ('complete -o default -C "{py} {script}" gpaw'
                  .format(py=sys.executable, script=script))


if __name__ == '__main__':
    # Path of the complete.py script:
    update(script, commands)



from __future__ import print_function
from ase import Atom
from ase.neb import SingleCalculatorNEB
from ase.optimize import FIRE
from gpaw import GPAW
from gpaw.cluster import Cluster

txt=None

mol = Cluster([Atom('H'),
               Atom('H',[1,0,0]),
               Atom('H',[.5,.5,.5])],
              cell = [2,2,2],
              pbc=True)

def set_calculators(all=False):
    c=GPAW(h=.3, convergence={'eigenstates':0.1, 
                              'energy' : 0.1,
                              'density' : 0.01}, 
           symmetry='off', txt=txt)
#    c = EMT()
    n = len(images)
    if not all:
        n -= 2
    neb.set_calculators([c] * n)

images = [mol]
for i in range(4):
    images.append(images[0].copy())
images[-1].positions[2, 1] = 2 - images[0].positions[2, 1]
neb = SingleCalculatorNEB(images)
neb.interpolate()
for image in images:
    print(image[2].position)
set_calculators(True)

dyn = FIRE(neb, trajectory='mep.traj')
dyn.insert_observer(set_calculators)
print(dyn.run(fmax=8.))



"""Make sure we get an exception when an atom is too close to the boundary."""
from ase import Atoms
from gpaw import GPAW
from gpaw.grid_descriptor import GridBoundsError
a = 4.0
x = 0.1
hydrogen = Atoms('H', [(x, x, x)],
                 cell=(a, a, a),
                 calculator=GPAW(maxiter=7))
try:
    e1 = hydrogen.get_potential_energy()
except GridBoundsError:
    pass
else:
    assert False



from __future__ import print_function
from ase import Atoms
from ase.io import read
from ase.test import must_raise
from gpaw import GPAW, FermiDirac
from gpaw.test import equal

# Check that we fail for non-3x3 cell:
with must_raise(ValueError):
    H = Atoms('H', calculator=GPAW())
    H.get_potential_energy()

a = 2.0
calc = GPAW(gpts=(12, 12, 12), txt='H.txt', occupations=FermiDirac(0.0))
H = Atoms('H',
          cell=(a, a, a),
          pbc=True,
          calculator=calc)
e0 = H.get_potential_energy()

H = read('H.txt')
equal(H.get_potential_energy(), e0, 1e-6)

energy_tolerance = 0.001
equal(e0, -6.5577, energy_tolerance)

print(calc.get_xc_functional())



import os

import numpy as np

from gpaw import GPAW, PW
from gpaw.mpi import world
from gpaw.test import equal, findpeak
from gpaw.response.df import DielectricFunction
from ase.build import bulk
from ase.units import Bohr, Hartree

# Comparing the plasmon peaks found in bulk sodium for two different
# atomic structures. Testing for idential plasmon peaks. Not using
# physical sodium cell.

a1 = bulk('Na')
a2 = bulk('Na')
a2.set_initial_magnetic_moments([[0.1, ]])
a1.calc = GPAW(mode=PW(300),
               kpts={'size': (8, 8, 8), 'gamma': True},
               parallel={'band': 1},
               txt='na_spinpaired.txt')
a2.calc = GPAW(mode=PW(300),
               kpts={'size': (8, 8, 8), 'gamma': True},
               parallel={'band': 1},
               txt='na_spinpol.txt')
a1.get_potential_energy()
a2.get_potential_energy()

# Use twice as many bands for expanded structure
a1.calc.diagonalize_full_hamiltonian(nbands=20)
a2.calc.diagonalize_full_hamiltonian(nbands=20)

a1.calc.write('intraband_spinpaired.gpw', 'all')
a2.calc.write('intraband_spinpolarized.gpw', 'all')

# Calculate the dielectric functions
if world.rank == 0:
    try:
        os.remove('intraband_spinpaired+0+0+0.pckl')
    except OSError:
        pass
    
df1 = DielectricFunction('intraband_spinpaired.gpw',
                         domega0=0.03,
                         ecut=10,
                         rate=0.1,
                         integrationmode='tetrahedron integration',
                         name='intraband_spinpaired',
                         txt='intraband_spinpaired_df.txt')

df1NLFCx, df1LFCx = df1.get_dielectric_function(direction='x')
df1NLFCy, df1LFCy = df1.get_dielectric_function(direction='y')
df1NLFCz, df1LFCz = df1.get_dielectric_function(direction='z')
wp1_vv = df1.chi0.plasmafreq_vv**0.5
wp1 = wp1_vv[0, 0]
if world.rank == 0:
    try:
        os.remove('intraband_spinpolarized+0+0+0.pckl')
    except OSError:
        pass

df2 = DielectricFunction('intraband_spinpolarized.gpw',
                         domega0=0.03,
                         ecut=10,
                         rate=0.1,
                         integrationmode='tetrahedron integration',
                         name='intraband_spinpolarized',
                         txt='intraband_spinpolarized_df.txt')

df2NLFCx, df2LFCx = df2.get_dielectric_function(direction='x')
df2NLFCy, df2LFCy = df2.get_dielectric_function(direction='y')
df2NLFCz, df2LFCz = df2.get_dielectric_function(direction='z')
wp2_vv = df2.chi0.plasmafreq_vv**0.5
wp2 = wp2_vv[0, 0]

# Compare plasmon frequencies and intensities
w_w = df1.chi0.omega_w

# Analytical Drude result
n = 1 / (a1.get_volume() * Bohr**-3)

wp = np.sqrt(4 * np.pi * n)

# From http://pubs.acs.org/doi/abs/10.1021/jp810808h
wpref = 5.71 / Hartree

equal(wp1, wp2, 1e-2)
equal(wp1, wp, 0.5)  # Use larger margin when comparing to Drude
equal(wp1, wpref, 0.1)

w1, I1 = findpeak(w_w, -(1. / df1LFCx).imag)
w2, I2 = findpeak(w_w, -(1. / df2LFCx).imag)
equal(w1, w2, 1e-2)
equal(I1, I2, 1e-1)

w1, I1 = findpeak(w_w, -(1. / df1LFCy).imag)
w2, I2 = findpeak(w_w, -(1. / df2LFCy).imag)
equal(w1, w2, 1e-2)
equal(I1, I2, 1e-1)

w1, I1 = findpeak(w_w, -(1. / df1LFCz).imag)
w2, I2 = findpeak(w_w, -(1. / df2LFCz).imag)
equal(w1, w2, 1e-2)
equal(I1, I2, 1e-1)



from __future__ import print_function
import os
import re
import numpy as np
from gpaw.atom.basis import BasisMaker
from gpaw.atom.basis import QuasiGaussian
from gpaw.atom.radialgd import EquidistantRadialGridDescriptor
from gpaw.atom.configurations import parameters, parameters_extra
from gpaw.basis_data import BasisFunction
from gpaw.mpi import world

# Module for generating basis sets that compose of usual basis sets
# augmented with Gaussian type orbital (GTO).
#
# GTOs are truncated and represented numerically.


def read_gbs(fname):
    """Read gbs file.

    This reads only the first element/atom from the file
    as separated with line beginning with '*'.
    """
    gto_k = []
    description = ''

    f = open(fname, 'r')
    line_i = f.readlines()
    f.close()

    i = 0
    Ni = len(line_i)
    while True:
        line = line_i[i].strip()
        if line == '' or line[0] == '*':
            pass
        elif line[0] == '!':
            description += '%s\n' % line[1:].strip()
        else:
            break
        i += 1
    description = description.strip()

    atom = line_i[i].strip().split()[0]
    i += 1
    while i < Ni:
        line = line_i[i]
        if line[0] == '*':
            break
        i += 1
        d = line.split()
        l = 'spdfghi'.index(d[0].lower())
        Nj = int(d[1])
        alpha_j = []
        coeff_j = []
        for _ in range(Nj):
            line = line_i[i]
            d = line.split()
            alpha = float(d[0])
            coeff = float(d[1])
            alpha_j.append(alpha)
            coeff_j.append(coeff)
            i += 1
        gto_k.append({'l': l, 'alpha_j': alpha_j, 'coeff_j': coeff_j})

    return atom, description, gto_k


def get_ngto(rgd, l, alpha, rcut):
    gaussian = QuasiGaussian(alpha, rcut)
    psi_g = gaussian(rgd.r_g) * rgd.r_g**l
    norm = np.sum(rgd.dr_g * (rgd.r_g * psi_g)**2)**.5
    psi_g /= norm
    return psi_g


def add_ngto(basis, l, alpha, tol, label):
    rgd = basis.get_grid_descriptor()
    rmax = rgd.r_g[-1]

    # Get NGTO with the initial (large) rcut=rmax
    psiref_g = get_ngto(rgd, l, alpha, rmax)

    # Make rcut smaller

    # Guess initial rcut where we are close to the tolerance
    i = np.where(psiref_g > tol)[0][-1]
    rcut = rgd.r_g[i]
    psi_g = get_ngto(rgd, l, alpha, rcut)
    err = np.max(np.absolute(psi_g - psiref_g))

    # Increase/decrease rcut to find the smallest rcut
    # that yields error within the tolerance
    if err > tol:
        # Increase rcut -> decrease err
        for i in range(i, len(rgd.r_g)):
            rcut = rgd.r_g[i]
            psi_g = get_ngto(rgd, l, alpha, rcut)
            err = np.max(np.absolute(psi_g - psiref_g))
            if err < tol:
                break
    else:
        # Decrease rcut -> increase err
        for i in range(i, 0, -1):
            rcut = rgd.r_g[i]
            psi_g = get_ngto(rgd, l, alpha, rcut)
            err = np.max(np.absolute(psi_g - psiref_g))
            if err > tol:
                i += 1
                break

    # Construct NGTO with the found rcut
    rcut = rgd.r_g[i]
    psi_g = get_ngto(rgd, l, alpha, rcut)

    # Change norm (maybe unnecessary)
    psi_g = psi_g[:(i + 1)] * 0.5

    # Create associated basis function
    bf = BasisFunction(None, l, rcut, psi_g, label)
    basis.bf_j.append(bf)


def do_nao_ngto_basis(atom, xc, naobasis, gbsfname, label, rmax=100.0):
    # Read Gaussians
    atomgbs, descriptiongbs, gto_k = read_gbs(gbsfname)
    assert atom == atomgbs

    # Generate nao basis
    assert naobasis == 'sz'

    # Choose basis sets without semi-core states XXXXXX
    if atom == 'Ag':
        label = '11.%s' % label
        p = parameters_extra
    else:
        p = parameters

    bm = BasisMaker(atom, label, run=False, gtxt=None, xc=xc)
    bm.generator.run(write_xml=False, use_restart_file=False, **p[atom])
    basis = bm.generate(1, 0, txt=None)

    # Increase basis function max radius
    h = basis.rgd.dr_g
    assert isinstance(h, float)
    assert basis.rgd.r_g[0] == 0.0
    N = int(rmax / h) + 1
    basis.rgd = EquidistantRadialGridDescriptor(h, N)

    # Add NGTOs
    tol = 0.001

    description = []
    msg = 'Augmented with NGTOs'
    description.append(msg)
    description.append('=' * len(msg))
    description.append('')
    msg = 'GTOs from file %s' % os.path.basename(gbsfname)
    description.append(msg)
    description.append('-' * len(msg))
    description.append(descriptiongbs)
    description.append('')
    description.append('NGTO truncation tolerance: %f' % tol)
    description.append('Functions: NGTO(l,alpha)')

    for gto in gto_k:
        l = gto['l']
        assert len(gto['alpha_j']) == 1, \
            'Only non-contracted GTOs supported'
        alpha = gto['alpha_j'][0]
        ngtolabel = 'NGTO(%s,%.7f)' % ('spdfghi'[l], alpha)
        description.append('    ' + ngtolabel)
        add_ngto(basis, l, alpha, tol, ngtolabel)

    basis.generatordata += '\n\n' + '\n'.join(description)

    basis.write_xml()


def main():
    xc = 'PBE'

    # Process all gbs files
    fname_i = [fname for fname in sorted(os.listdir('.'))
               if fname.endswith('.gbs')]
    for i, fname in enumerate(fname_i):
        if i % world.size != world.rank:
            continue
        m = re.match('(?P<atom>\w+)-(?P<label>NAO-(?P<nao>\w+)\+' +
                     'NGTO-N(?P<Nngto>\d+)).gbs', fname)
        if m is not None:
            if world.size > 1:
                print(world.rank, fname)
            else:
                print(fname)
            do_nao_ngto_basis(m.group('atom'), xc, m.group('nao'),
                              fname, m.group('label'))


if __name__ == '__main__':
    main()



from __future__ import print_function
import numpy as np
from gpaw.lfc import LocalizedFunctionsCollection as LFC
from gpaw.grid_descriptor import GridDescriptor, RadialGridDescriptor
from gpaw.spline import Spline
from gpaw.response.math_func import two_phi_planewave_integrals

# Initialize s, p, d (9 in total) wave and put them on grid
rc = 2.0
a = 2.5 * rc
n = 64
lmax = 2
b = 8.0
m = (lmax + 1)**2
gd = GridDescriptor([n, n, n], [a, a, a])
r = np.linspace(0, rc, 200)
g = np.exp(-(r / rc * b)**2)
splines = [Spline(l=l, rmax=rc, f_g=g) for l in range(lmax + 1)]
c = LFC(gd, [splines])
c.set_positions([(0.5, 0.5, 0.5)])
psi = gd.zeros(m)
d0 = c.dict(m)
if 0 in d0:
    d0[0] = np.identity(m)
c.add(psi, d0)

# Calculate on 3d-grid < phi_i | e**(-ik.r) | phi_j >
R_a = np.array([a/2,a/2,a/2])
rr = gd.get_grid_point_coordinates()
for dim in range(3):
    rr[dim] -= R_a[dim]

k_G = np.array([[11.,0.2,0.1],[10., 0., 10.]])
nkpt = k_G.shape[0]

d0 = np.zeros((nkpt,m,m), dtype=complex)
for i in range(m):
    for j in range(m):
        for ik in range(nkpt):
            k = k_G[ik]
            kk = np.sqrt(np.inner(k,k))
            kr = np.inner(k,rr.T).T
            expkr = np.exp(-1j * kr)
            d0[ik, i,j] = gd.integrate(psi[i] * psi[j] * expkr)

# Calculate on 1d-grid < phi_i | e**(-ik.r) | phi_j >
rgd = RadialGridDescriptor(r, np.ones_like(r) * r[1])
g = [np.exp(-(r / rc * b)**2)*r**l for l in range(lmax + 1)]
l_j = range(lmax + 1)
d1 = two_phi_planewave_integrals(k_G, rgd=rgd, phi_jg=g,
                                 phit_jg=np.zeros_like(g), l_j=l_j)

d1 = d1.reshape(nkpt, m, m)

for i in range(m):
    for j in range(m):
        for ik in range(nkpt):
            if np.abs(d0[ik,i,j] - d1[ik,i,j]) > 1e-10:
                print(i, j, d0[ik,i,j]- d1[ik,i,j])



from ase import Atom, Atoms
from gpaw import GPAW, Mixer
from gpaw.test import equal

# This test would be useful if it did anything with mixers that the
# other tests didn't.  Maybe verify that it uses mixing history
# correctly, or the long-range damping, or something.  Preferebly
# without a self-consistent calculation.

a = 2.7
bulk = Atoms([Atom('Li')], pbc=True, cell=(a, a, a))
k = 2
g = 16
calc = GPAW(gpts=(g, g, g), kpts=(k, k, k), nbands=2,
                  mixer=Mixer(nmaxold=5))
bulk.set_calculator(calc)
e = bulk.get_potential_energy()
niter = calc.get_number_of_iterations()
calc.write('Li.gpw')
calc2 = GPAW('Li.gpw')

energy_tolerance = 0.0001
equal(e, -1.20257, energy_tolerance)



# this test should coverage the save and restore of
# fermi-levels when using fixmagmom:
#
# yes, fermi-level-splitting sounds a little bit strange

from ase import Atoms
from gpaw import GPAW, FermiDirac, MixerSum
from gpaw.test import equal

calc = GPAW(occupations=FermiDirac(width=0.1, fixmagmom=True),
            mixer=MixerSum(beta=0.05, nmaxold=3, weight=50.0),
            convergence={'energy': 0.1, 'eigenstates': 1.5e-1,
                         'density': 1.5e-1})
atoms = Atoms('Cr', pbc=False)
atoms.center(vacuum=4)
mm = [1] * 1
mm[0] = 6.
atoms.set_initial_magnetic_moments(mm)
atoms.set_calculator(calc)
atoms.get_potential_energy()

ef1 = calc.occupations.get_fermi_levels_mean()
efsplit1 = calc.occupations.get_fermi_splitting()

ef3 = calc.occupations.get_fermi_levels()
calc.write('test.gpw')

# check number one: is the splitting value saved?
readtest = GPAW('test.gpw')
ef2 = readtest.occupations.get_fermi_levels_mean()
efsplit2 = readtest.occupations.get_fermi_splitting()

# numpy arrays
ef4 = readtest.occupations.get_fermi_levels()

# These values should be identic
equal(ef1, ef2, 1e-9)
equal(efsplit1, efsplit2, 1e-9)
equal(ef3.mean(), ef1, 1e-9)
equal(ef3.mean(), ef2, 1e-9)
equal(ef3.mean(), ef4.mean(), 1e-9)
equal(ef3[0] - ef3[1], ef4[0] - ef4[1], 1e-9)
equal(efsplit1, ef4[0] - ef4[1], 1e-9)



from __future__ import print_function
from ase import Atom, Atoms
from gpaw import GPAW
from gpaw.test import equal

a = 4.05
d = a / 2**0.5
bulk = Atoms([Atom('Al', (0, 0, 0)),
              Atom('Al', (0, 0, d))],
             cell=(4*d, 4*d, 2*d),
             pbc=1)
n = 16
calc = GPAW(gpts=(2*n, 2*n, 1*n),
            nbands=1*8,
            kpts=(1, 1, 4),
            convergence={'eigenstates': 2.3e-9},
            xc='LDA')
bulk.set_calculator(calc)
e2 = bulk.get_potential_energy()
niter2 = calc.get_number_of_iterations()

bulk = bulk.repeat((1, 1, 2))
bulk.set_calculator(calc)
calc.set(nbands=16, kpts=(1, 1, 2), gpts=(2*n, 2*n, 2*n))
e4 = bulk.get_potential_energy()
niter4 = calc.get_number_of_iterations()

# checks
energy_tolerance = 0.002
niter_tolerance = 0

print(e2, e4)
equal(e4 / 2, e2, 48e-6)
equal(e2, -3.41595, energy_tolerance)
equal(e4, -6.83191, energy_tolerance)



# Copyright (C) 2003  CAMP
# Copyright (C) 2010  Argonne National Laboratory
# Please see the accompanying LICENSE file for further information.

"""
Python wrapper functions for the C and Fortran packages:
Basic Linear Algebra Communication Subprogramcs (BLACS)
ScaLAPACK

See also:
http://www.netlib.org/blacs
and
http://www.netlib.org/scalapack
"""

import warnings

import _gpaw

switch_lu = {'U': 'L', 'L': 'U'}

def scalapack_tri2full(desc, array):
    """Write lower triangular part into upper triangular part of matrix.

    This function is a frightful hack, but we can improve the
    implementation later."""

    # Zero upper triangle:
    scalapack_zero(desc, array, 'U')
    buf = array.copy()
    # Set diagonal to zero in the copy:
    scalapack_set(desc, buf, alpha=0.0, beta=0.0, uplo='U')
    # Now transpose tmp_mm adding the result to the original matrix:
    pblas_tran(alpha=1.0, a_MN=buf,
               beta=1.0, c_NM=array,
               desca=desc, descc=desc)


def scalapack_zero(desca, a, uplo, ia=1, ja=1):
    """Zero the upper or lower half of a square matrix."""
    assert desca.gshape[0] == desca.gshape[1]
    p = desca.gshape[0] - 1
    if uplo == 'L':
        ia = ia + 1
    else:
        ja = ja + 1
    scalapack_set(desca, a, 0.0, 0.0, uplo, p, p, ia, ja)


def scalapack_set(desca, a, alpha, beta, uplo, m=None, n=None, ia=1, ja=1):
    """Set the diagonal and upper/lower triangular part of a.

    Set the upper or lower triangular part of a to alpha, and the diagonal
    of a to beta, where alpha and beta are real or complex numbers."""
    desca.checkassert(a)
    assert uplo in ['L', 'U']
    if m is None:
        m = desca.gshape[0]
    if n is None:
        n = desca.gshape[1]
    if not desca.blacsgrid.is_active():
        return
    _gpaw.scalapack_set(a, desca.asarray(), alpha, beta,
                        switch_lu[uplo], n, m, ja, ia)


def scalapack_diagonalize_dc(desca, a, z, w, uplo):
    """Diagonalize symmetric matrix using the divide & conquer algorithm.
    Orthogonal eigenvectors not guaranteed; no warning is provided.

    Solve the eigenvalue equation::

      A_nn Z_nn = w_N Z_nn

    Diagonalizes A_nn and writes eigenvectors to Z_nn.  Both A_nn
    and Z_nn must be compatible with desca descriptor.  Values in
    A_nn will be overwritten.

    Eigenvalues are written to the global array w_N in ascending order.

    The `uplo` flag can be either 'L' or 'U', meaning that the
    matrices are taken to be upper or lower triangular respectively.
    """
    desca.checkassert(a)
    desca.checkassert(z)
    # only symmetric matrices
    assert desca.gshape[0] == desca.gshape[1]
    assert uplo in ['L', 'U']
    if not desca.blacsgrid.is_active():
        return
    assert desca.gshape[0] == len(w)
    info = _gpaw.scalapack_diagonalize_dc(a, desca.asarray(),
                                          switch_lu[uplo], z, w)
    if info != 0:
        raise RuntimeError('scalapack_diagonalize_dc error: %d' % info)


def scalapack_diagonalize_ex(desca, a, z, w, uplo, iu=None):
    """Diagonalize symmetric matrix using the bisection and inverse
    iteration algorithm. Re-orthogonalization of eigenvectors
    is an issue for tightly clustered eigenvalue problems; it
    requires substantial memory and is not scalable. See ScaLAPACK
    pdsyevx.f routine for more information.

    Solve the eigenvalue equation::

      A_nn Z_nn = w_N Z_nn

    Diagonalizes A_nn and writes eigenvectors to Z_nn.  Both A_nn
    and Z_nn must be compatible with desca descriptor.  Values in
    A_nn will be overwritten.

    Eigenvalues are written to the global array w_N in ascending order.

    The `uplo` flag can be either 'L' or 'U', meaning that the
    matrices are taken to be upper or lower triangular respectively.

    The `iu` specifies how many eigenvectors and eigenvalues to compute.
    """
    desca.checkassert(a)
    desca.checkassert(z)
    # only symmetric matrices
    assert desca.gshape[0] == desca.gshape[1]
    if iu is None: # calculate all eigenvectors and eigenvalues
        iu = desca.gshape[0]
    assert 1 < iu <= desca.gshape[0]
    # still need assert for eigenvalues
    assert uplo in ['L', 'U']
    if not desca.blacsgrid.is_active():
        return
    assert desca.gshape[0] == len(w)
    if (desca.blacsgrid.myrow, desca.blacsgrid.mycol) == (0, 0):
        message = 'scalapack_diagonalize_ex may have a buffer ' \
            'overflow, use scalapack_diagonalize_dc instead'
        warnings.warn(message, RuntimeWarning)
    info = _gpaw.scalapack_diagonalize_ex(a, desca.asarray(),
                                          switch_lu[uplo],
                                          iu, z, w)
    if info != 0:
        # 0 means you are OK
        raise RuntimeError('scalapack_diagonalize_ex error: %d' % info)

def scalapack_diagonalize_mr3(desca, a, z, w, uplo, iu=None):
    """Diagonalize symmetric matrix using the MRRR algorithm.

    Solve the eigenvalue equation::

      A_nn Z_nn = w_N Z_nn

    Diagonalizes A_nn and writes eigenvectors to Z_nn.  Both A_nn
    and Z_nn must be compatible with this desca descriptor.  Values in
    A_nn will be overwritten.

    Eigenvalues are written to the global array w_N in ascending order.

    The `uplo` flag can be either 'L' or 'U', meaning that the
    matrices are taken to be upper or lower triangular respectively.

    The `iu` specifies how many eigenvectors and eigenvalues to compute.
    """
    desca.checkassert(a)
    desca.checkassert(z)
    # only symmetric matrices
    assert desca.gshape[0] == desca.gshape[1]
    if iu is None: # calculate all eigenvectors and eigenvalues
        iu = desca.gshape[0]
    assert 1 < iu <= desca.gshape[0]
    # stil need assert for eigenvalues
    assert uplo in ['L', 'U']
    if not desca.blacsgrid.is_active():
        return
    assert desca.gshape[0] == len(w)
    info = _gpaw.scalapack_diagonalize_mr3(a, desca.asarray(),
                                           switch_lu[uplo],
                                           iu, z, w)
    if info != 0:
        raise RuntimeError('scalapack_diagonalize_mr3 error: %d' % info)

def scalapack_general_diagonalize_dc(desca, a, b, z, w, uplo):
    """Diagonalize symmetric matrix using the divide & conquer algorithm.
    Orthogonal eigenvectors not guaranteed; no warning is provided.

    Solve the generalized eigenvalue equation::

      A_nn Z_nn = w_N B_nn Z_nn

    B_nn is assumed to be positivde definite. Eigenvectors written to Z_nn.
    Both A_nn, B_nn and Z_nn must be compatible with desca descriptor.
    Values in A_nn and B_nn will be overwritten.

    Eigenvalues are written to the global array w_N in ascending order.

    The `uplo` flag can be either 'L' or 'U', meaning that the
    matrices are taken to be upper or lower triangular respectively.
    """
    desca.checkassert(a)
    desca.checkassert(b)
    desca.checkassert(z)
    # only symmetric matrices
    assert desca.gshape[0] == desca.gshape[1]
    assert uplo in ['L', 'U']
    if not desca.blacsgrid.is_active():
        return
    assert desca.gshape[0] == len(w)
    info = _gpaw.scalapack_general_diagonalize_dc(a, desca.asarray(),
                                          switch_lu[uplo], b, z, w)
    if info != 0:
        raise RuntimeError('scalapack_general_diagonalize_dc error: %d' % info)

def scalapack_general_diagonalize_ex(desca, a, b, z, w, uplo, iu=None):
    """Diagonalize symmetric matrix using the bisection and inverse
    iteration algorithm. Re-orthogonalization of eigenvectors
    is an issue for tightly clustered eigenvalue problems; it
    requires substantial memory and is not scalable. See ScaLAPACK
    pdsyevx.f routine for more information.

    Solves the eigenvalue equation::

      A_nn Z_nn = w_N B_nn Z_nn

    B_nn is assumed to be positivde definite. Eigenvectors written to Z_nn.
    Both A_nn, B_nn and Z_nn must be compatible with desca descriptor.
    Values in A_nn and B_nn will be overwritten.

    Eigenvalues are written to the global array w_N in ascending order.

    The `uplo` flag can be either 'L' or 'U', meaning that the
    matrices are taken to be upper or lower triangular respectively.

    The `iu` specifies how many eigenvectors and eigenvalues to compute.
    """
    desca.checkassert(a)
    desca.checkassert(b)
    desca.checkassert(z)
    # only symmetric matrices
    assert desca.gshape[0] == desca.gshape[1]
    if iu is None: # calculate all eigenvectors and eigenvalues
        iu = desca.gshape[0]
    assert 1 < iu <= desca.gshape[0]
    # still need assert for eigenvalues
    assert uplo in ['L', 'U']
    if not desca.blacsgrid.is_active():
        return
    assert desca.gshape[0] == len(w)
    if (desca.blacsgrid.myrow, desca.blacsgrid.mycol) == (0, 0):
        message = 'scalapack_general_diagonalize_ex may have a buffer ' \
            'overflow, use scalapack_general_diagonalize_dc instead'
        warnings.warn(message, RuntimeWarning)
    info = _gpaw.scalapack_general_diagonalize_ex(a, desca.asarray(),
                                                  switch_lu[uplo],
                                                  iu, b, z, w)
    if info != 0:
        # 0 means you are OK
        raise RuntimeError('scalapack_general_diagonalize_ex error: %d' % info)

def scalapack_general_diagonalize_mr3(desca, a, b, z, w, uplo, iu=None):
    """Diagonalize symmetric matrix using the MRRR algorithm.

    Solve the generalized eigenvalue equation::

      A_nn Z_nn = w_N B_nn Z_nn

    B_nn is assumed to be positivde definite. Eigenvectors written to Z_nn.
    Both A_nn, B_nn and Z_nn must be compatible with desca descriptor.
    Values in A_nn and B_nn will be overwritten.

    Eigenvalues are written to the global array w_N in ascending order.

    The `uplo` flag can be either 'L' or 'U', meaning that the
    matrices are taken to be upper or lower triangular respectively.

    The `iu` specifies how many eigenvectors and eigenvalues to compute.
    """
    desca.checkassert(a)
    desca.checkassert(b)
    desca.checkassert(z)
    # only symmetric matrices
    assert desca.gshape[0] == desca.gshape[1]
    if iu is None: # calculate all eigenvectors and eigenvalues
        iu = desca.gshape[0]
    assert 1 < iu <= desca.gshape[0]
    # still need assert for eigenvalues
    assert uplo in ['L', 'U']
    if not desca.blacsgrid.is_active():
        return
    assert desca.gshape[0] == len(w)
    info = _gpaw.scalapack_general_diagonalize_mr3(a, desca.asarray(),
                                                   switch_lu[uplo],
                                                   iu, b, z, w)
    if info != 0:
        raise RuntimeError('scalapack_general_diagonalize_mr3 error: %d' % info)

def scalapack_inverse_cholesky(desca, a, uplo):
    """Perform Cholesky decomposin followed by an inversion
    of the resulting triangular matrix.

    Only the upper or lower half of the matrix a will be
    modified; the other half is zeroed out.

    The `uplo` flag can be either 'L' or 'U', meaning that the
    matrices are taken to be upper or lower triangular respectively.
    """
    desca.checkassert(a)
    # only symmetric matrices
    assert desca.gshape[0] == desca.gshape[1]
    assert uplo in ['L', 'U']
    if not desca.blacsgrid.is_active():
        return
    info = _gpaw.scalapack_inverse_cholesky(a, desca.asarray(),
                                            switch_lu[uplo])
    if info != 0:
        raise RuntimeError('scalapack_inverse_cholesky error: %d' % info)


def scalapack_inverse(desca, a, uplo):
    """Perform a hermitian matrix inversion.

    """
    desca.checkassert(a)
    # only symmetric matrices
    assert desca.gshape[0] == desca.gshape[1]
    assert uplo in ['L', 'U']
    if not desca.blacsgrid.is_active():
        return
    info = _gpaw.scalapack_inverse(a, desca.asarray(), switch_lu[uplo])
    if info != 0:
        raise RuntimeError('scalapack_inverse error: %d' % info)

def scalapack_solve(desca, descb, a, b):
    """Perform general matrix solution to Ax=b. Result will be replaces with b.
       Equivalent to numpy.linalg.solve(a, b.T.conjugate()).T.conjugate()

    """
    desca.checkassert(a)
    descb.checkassert(b)
    # only symmetric matrices
    assert desca.gshape[0] == desca.gshape[1]
    # valid equation
    assert desca.gshape[1] == descb.gshape[1]

    if not desca.blacsgrid.is_active():
        return
    info = _gpaw.scalapack_solve(a.T, desca.asarray(), b.T, descb.asarray())
    if info != 0:
        raise RuntimeError('scalapack_solve error: %d' % info)


def pblas_tran(alpha, a_MN, beta, c_NM, desca, descc):
    """C <- beta C + alpha A.T.

    See also pdtran from PBLAS."""
    desca.checkassert(a_MN)
    descc.checkassert(c_NM)
    M, N = desca.gshape
    assert N, M == descc.gshape
    _gpaw.pblas_tran(N, M, alpha, a_MN, beta, c_NM,
                     desca.asarray(), descc.asarray())


def pblas_hemm(alpha, a_MK, b_KN, beta, c_MN, desca, descb, descc,
               side='L', uplo='L'):
    # Hermitean matrix multiply, only lower or upper diagonal of a_MK
    # is used. By default, C = beta*C + alpha*A*B
    # Executes PBLAS method pzhemm for complex and pdsymm for real matrices.
    desca.checkassert(a_MK)
    descb.checkassert(b_KN)
    descc.checkassert(c_MN)
    assert side in ['R','L'] and uplo in ['L','U']
    M, Ka = desca.gshape
    Kb, N = descb.gshape
    if side=='R':
        Kb, N = N, Kb

    if not desca.blacsgrid.is_active():
        return
    fortran_side = {'L':'R', 'R':'L'}
    fortran_uplo = {'U':'L', 'L':'U'}
    if side=='R':
        M, N = N, M

    _gpaw.pblas_hemm(fortran_side[side], fortran_uplo[uplo],
                     N, M, alpha, a_MK.T, b_KN.T, beta, c_MN.T,
                     desca.asarray(), descb.asarray(), descc.asarray())


def pblas_gemm(alpha, a_MK, b_KN, beta, c_MN, desca, descb, descc,
               transa='N', transb='N'):
    desca.checkassert(a_MK)
    descb.checkassert(b_KN)
    descc.checkassert(c_MN)
    assert transa in ['N', 'T', 'C'] and transb in ['N', 'T', 'C']
    M, Ka = desca.gshape
    Kb, N = descb.gshape

    if transa == 'T':
        M, Ka = Ka, M
    if transb == 'T':
        Kb, N = N, Kb
    Mc, Nc = descc.gshape

    assert Ka == Kb
    assert M == Mc
    assert N == Nc

    # trans = transa + transb

    """
    if transb == 'N':
        assert desca.gshape[1] == descb.gshape[0]
        assert desca.gshape[0] == descc.gshape[0]
        assert descb.gshape[1] == descc.gshape[1]
    if transb == 'T':
        N, Kb = Kb, N
        #assert desca.gshape[1] == descb.gshape[1]
        assert desca.gshape[0] == descc.gshape[0]
        assert descb.gshape[0] == descc.gshape[1]

    if trans == 'NN':
        assert desca.gshape[1] == descb.gshape[0]
        assert desca.gshape[0] == descc.gshape[0]
        assert descb.gshape[1] == descc.gshape[1]
    elif transa == 'T':
        M, Ka = Ka, M
        assert desca.gshape[1] == descc.gshape[0]
    if transb == 'N':
        assert descb.gshape[1] == descc.gshape[1]
    elif transb == 'T':
        assert descb.gshape[1] == descc.gshape[1]
    assert Ka == Kb
    #assert transa == 'N' # XXX remember to implement 'T'
    _gpaw.pblas_gemm(N, M, Ka, alpha, b_KN.T, a_MK.T, beta, c_MN.T,
    """
    # assert transa == 'N' # XXX remember to implement 'T'
    if not desca.blacsgrid.is_active():
        return
    _gpaw.pblas_gemm(N, M, Ka, alpha, b_KN.T, a_MK.T, beta, c_MN.T,
                     descb.asarray(), desca.asarray(), descc.asarray(),
                     transb, transa)


def pblas_simple_gemm(desca, descb, descc, a_MK, b_KN, c_MN,
                      transa='N', transb='N'):
    alpha = 1.0
    beta = 0.0
    pblas_gemm(alpha, a_MK, b_KN, beta, c_MN, desca, descb, descc,
               transa, transb)


def pblas_simple_hemm(desca, descb, descc, a_MK, b_KN, c_MN, side='L', uplo='L'):
    alpha = 1.0
    beta = 0.0
    pblas_hemm(alpha, a_MK, b_KN, beta, c_MN, desca, descb, descc, side, uplo)

def pblas_gemv(alpha, a, x, beta, y, desca, descx, descy,
               transa='T'):
    desca.checkassert(a)
    descx.checkassert(x)
    descy.checkassert(y)
    M, N = desca.gshape
    # XXX transa = 'N' not implemented
    assert transa in ['T', 'C']
    assert desca.gshape[0] == descy.gshape[0]
    assert desca.gshape[1] == descx.gshape[0]
    assert descx.gshape[1] == descy.gshape[1]
    if not desca.blacsgrid.is_active():
        return
    _gpaw.pblas_gemv(N, M, alpha,
                     a, x, beta, y,
                     desca.asarray(),
                     descx.asarray(),
                     descy.asarray(),
                     transa)


def pblas_simple_gemv(desca, descx, descy, a, x, y):
    alpha = 1.0
    beta = 0.0
    pblas_gemv(alpha, a, x, beta, y, desca, descx, descy)


def pblas_r2k(alpha, a_NK, b_NK, beta, c_NN, desca, descb, descc,
                uplo='U'):
    if not desca.blacsgrid.is_active():
        return
    desca.checkassert(a_NK)
    descb.checkassert(b_NK)
    descc.checkassert(c_NN)
    assert descc.gshape[0] == descc.gshape[1] # symmetric matrix
    assert desca.gshape == descb.gshape # same shape
    assert uplo in ['L', 'U']
    N = descc.gshape[0] # order of C
    # K must take into account implicit tranpose due to C ordering
    K = desca.gshape[1] # number of columns of A and B
    _gpaw.pblas_r2k(N, K, alpha, a_NK, b_NK, beta, c_NN,
                    desca.asarray(),
                    descb.asarray(),
                    descc.asarray(),
                    uplo)


def pblas_simple_r2k(desca, descb, descc, a, b, c, uplo='U'):
    alpha = 1.0
    beta = 0.0
    pblas_r2k(alpha, a, b, beta, c,
                desca, descb, descc, uplo)


def pblas_rk(alpha, a_NK, beta, c_NN, desca, descc,
             uplo='U'):
    if not desca.blacsgrid.is_active():
        return
    desca.checkassert(a_NK)
    descc.checkassert(c_NN)
    assert descc.gshape[0] == descc.gshape[1] # symmetrix matrix
    assert uplo in ['L', 'U']
    N = descc.gshape[0] # order of C
    # K must take into account implicit tranpose due to C ordering
    K = desca.gshape[1] # number of columns of A
    _gpaw.pblas_rk(N, K, alpha, a_NK, beta, c_NN,
                    desca.asarray(),
                    descc.asarray(),
                    uplo)


def pblas_simple_rk(desca, descc, a, c):
    alpha = 1.0
    beta = 0.0
    pblas_rk(alpha, a, beta, c,
             desca, descc)




from __future__ import print_function
from gpaw.mpi import world
import numpy as np
from gpaw.lfc import LFC
from gpaw.analyse.observers import Observer
from math import sqrt, pi

#TODO: Rename this file

class VRespCollector(Observer):
    def __init__(self, filename, lcao):
        Observer.__init__(self)
        self.lcao = lcao
        self.filename = filename

    def update(self):
        x_sg = self.lcao.hamiltonian.xc.xcs['RESPONSE'].vt_sG
        x_sg = self.lcao.density.gd.collect(x_sg, broadcast=False)
        iter = self.niter
        if world.rank==0:
            fname = self.filename+"."+str(iter)+".vresp"
            x_sg.astype(np.float32).tofile(fname)

class DensityCollector(Observer):
    def __init__(self, filename, lcao, ranges_str='full'):
        Observer.__init__(self)
        self.lcao = lcao
        self.filename = filename
        self.ranges = None
        print("ranges-str", ranges_str)
        self.ranges_str = ranges_str

    def update(self):
        if self.ranges is None: # First time
            self.ranges = []
            self.nbands = self.lcao.wfs.bd.nbands
            start = 0
            if self.ranges_str != 'full':
                for rng in self.ranges_str.split(','):
                    print("rng", rng)
                    rng = eval(rng)
                    self.ranges.append(range(start, rng))
                    start = rng
            self.ranges.append(range(start, self.nbands))
            print(self.ranges)
            self.ghat = LFC(self.lcao.wfs.gd, [setup.ghat_l for setup in self.lcao.density.setups],
                            integral=sqrt(4 * pi), forces=False)
            self.ghat.set_positions(self.lcao.wfs.spos_ac)

            # Clear files
            for rid,rng in enumerate(self.ranges):
                f = open(self.filename+'.'+str(rid)+'.density','w')
                print("# Density file", file=f)
                N_c = self.lcao.wfs.gd.N_c
                print(N_c[0], N_c[1], N_c[2], file=f)
                print("# This header is 10 lines long, then double precision binary data starts.", file=f)
                for i in range(7):
                    print("#", file=f)
                f.close()

        #self.lcao.timer.start('Dump density')
        for rid,rng in enumerate(self.ranges):
            assert len(self.lcao.wfs.kpt_u) == 1
            f_un = [ self.lcao.wfs.kpt_u[0].f_n.copy() ]
            for n in range(self.lcao.wfs.bd.nbands):
                band_rank, myn = self.lcao.wfs.bd.who_has(n)
                if self.lcao.wfs.bd.rank == band_rank:
                    if n not in rng:
                        f_un[0][myn] = 0.0
            n_sG = self.lcao.wfs.gd.zeros(1)
            self.lcao.wfs.add_to_density_from_k_point_with_occupation(n_sG,
                          self.lcao.wfs.kpt_u[0], f_un[0])

            self.lcao.wfs.kptband_comm.sum(n_sG)

            D_asp={}
            for a in self.lcao.density.D_asp:
                ni = self.lcao.density.setups[a].ni
                D_asp[a] = np.zeros((1, ni * (ni + 1) // 2))
            self.lcao.wfs.calculate_atomic_density_matrices_with_occupation(D_asp, f_un)
            Q_aL = {}
            for a, D_sp in D_asp.items():
                Q_aL[a] = np.dot(D_sp.sum(0),
                                   self.lcao.density.setups[a].Delta_pL)
            self.ghat.add(n_sG, Q_aL)
            n_sg = self.lcao.wfs.gd.collect(n_sG, broadcast=False)
            if world.rank == 0:
                f = open(self.filename+'.'+str(rid)+'.density','a+')
                #n_sg.astype(np.float32).tofile(f)
                #print "max", np.max(n_sg), np.min(n_sg)
                n_sg.tofile(f)
                f.close()
                s = n_sg.shape
                f = open(self.filename+'.info','w')
                print(s[0],s[1],s[2], file=f)
                f.close()

# TODO: Remove
"""
class ObsoleteSplitDensityCollector(Observer):
    def __init__(self, filename, lcao, splitstr):
        Observer.__init__(self)
        self.lcao = lcao
        #self.lcao.timer.start('Split density init')
        self.filename = filename

        # Density arrays, files, and occupation numbers
        self.n_xsg = []
        self.f_xn = []

        #for i in range(2):
        #    if world.rank == 0:
        #        self.f_x.append(open(filename+'.'+str(i)+'.density','w'))
        #    else:
        #        self.f_x.append(None)
        for i in range(2):
            self.n_xsg.append(lcao.density.gd.zeros(1))
        f_n = lcao.wfs.kpt_u[0].f_n.copy()
        f2_n = lcao.wfs.kpt_u[0].f_n.copy()
        for n in range(lcao.wfs.bd.nbands):
            band_rank, myn = lcao.wfs.bd.who_has(n)
            if lcao.wfs.bd.rank == band_rank:
                if n < nsplit:
                    f_n[myn] = 0.0
                else:
                    f2_n[myn] = 0.0
        self.f_xn.append(f_n)
        self.f_xn.append(f2_n)
        #self.lcao.timer.start('Split density create LFC')
        #print "Creating lfc"
        #print "LFC created"
        self.lcao.timer.stop('Split density init')

        self.update()

    def update(self):
        self.lcao.timer.start('Split density dump')
        iter = self.niter
        if iter % 5 != 0:
            return
        start = time.time()
        kpt = self.lcao.wfs.kpt_u[0]
        for i, (f_n, n_sg) in enumerate(zip(self.f_xn, self.n_xsg)):
            n_sg[:] = 0.0 # XXX n_sg just temporary here
            self.lcao.wfs.add_to_density_from_k_point_with_occupation(n_sg, kpt, f_n)
            D_asp={}
            for a in self.lcao.density.D_asp:
                ni = self.lcao.density.setups[a].ni
                D_asp[a] = np.zeros((1, ni * (ni + 1) // 2))
            self.lcao.wfs.calculate_atomic_density_matrices_with_occupation(D_asp, f_n)
            for a, D_sp in D_asp.items():
                Q_aL = np.dot(D_sp.sum(0),
                                   self.lcao.density.setups[a].Delta_pL)
                qfile = open("%s.%i.atom.%d" % (self.filename,i,a),'a+')
                Q_aL.astype(np.float32).tofile(qfile)
                qfile.close()
            if self.lcao.wfs.bd.comm.rank == 0:
                gd = self.lcao.wfs.gd
                fname = self.filename+"."+str(i)+".%.5f.%d.%d.%d-%d.%d.%d.density" % (self.lcao.time, gd.beg_c[0], gd.beg_c[1], gd.beg_c[2], gd.end_c[0], gd.end_c[1], gd.end_c[2])
                n_sg.astype(np.float32).tofile(fname)
        took = time.time()-start
        if world.rank == 0:
            print("Density dump took:", took, "s")
        self.lcao.timer.stop('Split density dump')


atoms = read('../geometries/ag13.traj')
atoms.center(vacuum=5)

calc = LCAOTDDFT(mode='lcao', dtype=complex, width=0.01, basis='LDA.dz+5p', xc='LDA', h=0.3,
                 mixer=Mixer(0.1,4,weight=100))
atoms.set_calculator(calc)
atoms.get_potential_energy()
calc.absorption_kick([0.001,0.00,0.00])
split = SplitDensityCollector('ag13', calc, 13*5)
calc.attach(split, 1)
calc.propagate(10, 1000)

"""



from __future__ import print_function

from ase import Atoms
from gpaw import GPAW
from gpaw.xc.sic import SIC

a = 7.0
atom = Atoms('N', magmoms=[3], cell=(a, a, a))
molecule = Atoms('N2', positions=[(0, 0, 0), (0, 0, 1.14)], cell=(a, a, a))
atom.center()
molecule.center()

calc = GPAW(xc=SIC(),
            eigensolver='rmm-diis',
            h=0.17,
            txt='n2.sic.new3b.txt',
            setups='hgh')

atom.set_calculator(calc)
e1 = atom.get_potential_energy()

molecule.set_calculator(calc)
e2 = molecule.get_potential_energy()
F_ac = molecule.get_forces()
print(2 * e1 - e2)
print(F_ac)



# fails with On entry to ZGEMV parameter number 8 had an illegal value

from ase.build import molecule
from gpaw import GPAW
from gpaw.wavefunctions.pw import PW

m = molecule('H')
m.center(vacuum=2.0)
m.set_calculator(GPAW(mode=PW(), eigensolver='cg'))
m.get_potential_energy()




import numpy as np
from gpaw import debug
from gpaw.mpi import world, serial_comm, _Communicator, SerialCommunicator

even_comm = world.new_communicator(np.arange(0, world.size, 2))
if world.size > 1:
    odd_comm = world.new_communicator(np.arange(1, world.size, 2))
else:
    odd_comm = None

if world.rank % 2 == 0:
    assert odd_comm is None
    comm = even_comm
else:
    assert even_comm is None
    comm = odd_comm

hasmpi = False
try:
    import _gpaw
    hasmpi = hasattr(_gpaw, 'Communicator') and world.size > 1
except (ImportError, AttributeError):
    pass

assert world.parent is None
assert comm.parent is world
if hasmpi:
    assert comm.parent.get_c_object() is world.get_c_object()
    assert comm.get_c_object().parent is world.get_c_object()

commranks = np.arange(world.rank % 2, world.size, 2)
assert np.all(comm.get_members() == commranks)
assert comm.get_members()[comm.rank] == world.rank

subcomm = comm.new_communicator(np.array([comm.rank]))
assert subcomm.parent is comm
assert subcomm.rank == 0 and subcomm.size == 1
assert subcomm.get_members().item() == comm.rank

if debug:
    assert isinstance(world, _Communicator)
    assert isinstance(comm, _Communicator)
    assert isinstance(subcomm, _Communicator)
elif world is serial_comm:
    assert isinstance(world, SerialCommunicator)
    assert isinstance(comm, SerialCommunicator)
    assert isinstance(subcomm, SerialCommunicator)
elif hasmpi:
    assert isinstance(world, _gpaw.Communicator)
    assert isinstance(comm, _gpaw.Communicator)
    assert isinstance(subcomm, _gpaw.Communicator)



from __future__ import print_function
import numpy as np
from ase.build import bulk
from gpaw import GPAW
from gpaw.response.df import DielectricFunction
from gpaw.response.bse import BSE
from gpaw.test import findpeak, equal

GS = 1
df = 1
bse = 1
check_spectrum = 1

if GS:
    a = 4.043
    atoms = bulk('Al', 'fcc', a=a)
    calc = GPAW(mode='pw',
                kpts={'size': (4, 4, 4), 'gamma': True},
                xc='LDA',
                nbands=4,
                convergence={'bands': 'all'})
    
    atoms.set_calculator(calc)
    atoms.get_potential_energy()
    calc.write('Al.gpw', 'all')

q_c = np.array([0.25, 0.0, 0.0])
w_w = np.linspace(0, 24, 241)
eta = 0.2
ecut = 50
if bse:
    bse = BSE('Al.gpw',
              valence_bands=range(4),
              conduction_bands=range(4),
              mode='RPA',
              nbands=4,
              ecut=ecut,
              write_h=False,
              write_v=False,
              )
    bse_w = bse.get_eels_spectrum(filename=None,
                                  q_c=q_c,
                                  w_w=w_w,
                                  eta=eta)[1]
    
if df:
    df = DielectricFunction(calc='Al.gpw',
                            frequencies=w_w,
                            eta=eta,
                            ecut=ecut,
                            hilbert=False)
    df_w = df.get_eels_spectrum(q_c=q_c, filename=None)[1]

if check_spectrum:
    w_ = 15.1423
    I_ = 25.4359
    wbse, Ibse = findpeak(w_w, bse_w)
    wdf, Idf = findpeak(w_w, df_w)
    equal(wbse, w_, 0.01)
    equal(wdf, w_, 0.01)
    equal(Ibse, I_, 0.1)
    equal(Idf, I_, 0.1)



from gpaw.transformers import Transformer
import numpy.random as ra
from gpaw.grid_descriptor import GridDescriptor

p = 0
n = 20
gd1 = GridDescriptor((n, n, n), (8.0, 8.0, 8.0), pbc_c=p)
a1 = gd1.zeros()
ra.seed(8)
a1[:] = ra.random(a1.shape)
gd2 = gd1.refine()
a2 = gd2.zeros()
i = Transformer(gd1, gd2).apply
i(a1, a2)
assert abs(gd1.integrate(a1) - gd2.integrate(a2)) < 1e-10
r = Transformer(gd2, gd1).apply
a2[0] = 0.0
a2[:, 0] = 0.0
a2[:, :, 0] = 0.0
a2[-1] = 0.0
a2[:, -1] = 0.0
a2[:, :, -1] = 0.0
r(a2, a1)
assert abs(gd1.integrate(a1) - gd2.integrate(a2)) < 1e-10



"""Test calculation for bare proton.

Also, its interaction with an external potential in the form of a point charge
is tested.
"""
import numpy as np
from ase import Atoms
from ase.units import Bohr, Hartree
from gpaw import GPAW
from gpaw.external import PointChargePotential

a = 4.5
H = Atoms('H', [(a / 2, a / 2, a / 2)],
          pbc=0,
          cell=(a, a, a))
calc = GPAW(nbands=1, h=0.2, charge=1, txt='H.txt')
H.set_calculator(calc)
e0 = H.get_potential_energy()
assert abs(e0 + calc.get_reference_energy()) < 0.014

# Test the point charge potential with a smooth cutoff:
pcp = PointChargePotential([-1], rc2=5.5, width=1.5)
calc.set(external=pcp)
E = []
F = []
D = np.linspace(2, 6, 30)
for d in D:
    pcp.set_positions([[a / 2, a / 2, a / 2 + d]])
    e = H.get_potential_energy() - e0
    f1 = H.get_forces()
    f2 = pcp.get_forces(calc)
    eref = -1 / d * Bohr * Hartree
    print(d, e, eref, abs(f1 + f2).max())
    if d < 4.0:
        error = e + 1 / d * Bohr * Hartree
        assert abs(error) < 0.01, error
    assert abs(f1 + f2).max() < 0.01
    E.append(e)
    F.append(f1[0, 2])

E = np.array(E)
FF = (E[2:] - E[:-2]) / (D[2] - D[0])  # numerical force
print(abs(F[1:-1] - FF).max())
assert abs(F[1:-1] - FF).max() < 0.1

if not True:
    import matplotlib.pyplot as plt
    plt.plot(D, E)
    plt.plot(D, -1 / D * Bohr * Hartree)
    plt.plot(D, F)
    plt.plot(D[1:-1], FF)
    plt.show()



# Test of the handling of degenerate bands in response code
from ase import Atoms
from gpaw import GPAW, PW
from gpaw.response.df import DielectricFunction
from gpaw.test import findpeak, equal
import numpy as np


def get_hydrogen_chain_dielectric_function(NH, NK):
    a = Atoms('H', cell=[1, 1, 1], pbc=True)
    a.center()
    a = a.repeat((1, 1, NH))
    a.calc = GPAW(mode=PW(200, force_complex_dtype=True),
                  kpts={'size': (1, 1, NK), 'gamma': True},
                  parallel={'band': 1},
                  gpts=(10, 10, 10 * NH))
    a.get_potential_energy()
    a.calc.diagonalize_full_hamiltonian(nbands=2 * NH)
    a.calc.write('H_chain.gpw', 'all')

    DF = DielectricFunction('H_chain.gpw', ecut=1e-3, hilbert=False,
                            omega2=np.inf, intraband=False)
    eps_NLF, eps_LF = DF.get_dielectric_function(direction='z')
    omega_w = DF.get_frequencies()
    return omega_w, eps_LF

NH_i = [2**n for n in [0, 4]]
NK_i = [2**n for n in [6, 2]]

opeak_old = np.nan
peak_old = np.nan

for NH, NK in zip(NH_i, NK_i):
    omega_w, eps_w = get_hydrogen_chain_dielectric_function(NH, NK)
    eels_w = -(1. / eps_w).imag
    opeak, peak = findpeak(omega_w, eels_w)

    # Test for consistency
    if not np.isnan(opeak_old):
        equal(opeak, opeak_old, tolerance=1e-3)
        equal(peak, peak_old, tolerance=1e-3)
    opeak_old = opeak
    peak_old = peak



from ase import Atoms
from gpaw import GPAW, PW
from gpaw.mpi import serial_comm
from gpaw.test import equal
from gpaw.xc.rpa_correlation_energy import RPACorrelation

ecut = 50

He = Atoms('He')
He.center(vacuum=1.0)

calc = GPAW(mode=PW(force_complex_dtype=True),
            xc='PBE',
            communicator=serial_comm)
He.set_calculator(calc)
He.get_potential_energy()
calc.diagonalize_full_hamiltonian()

rpa = RPACorrelation(calc)
C6_rpa, C6_0 = rpa.get_C6_coefficient(ecut=ecut,
                                      direction=2)

equal(C6_0, 1.772, 0.01)
equal(C6_rpa, 1.387, 0.01)



import numpy as np
from gpaw import GPAW, Mixer
from gpaw.mpi import world
from ase.build import molecule

# Place one atom next to cell boundary, then check that reuse_wfs
# works correctly when atom is subsequently displaced across the
# boundary, i.e., that the kpoint phases of the PAW correction
# are handled correctly when unprojecting/reprojecting the wavefunctions.


def check(reuse):
    atoms = molecule('H2')
    atoms.pbc = 1
    atoms.center(vacuum=1.5)
    atoms.positions -= atoms.positions[1]
    dz = 1e-2
    atoms.positions[:, 2] += dz

    calc = GPAW(mode='pw',
                txt='gpaw.txt',
                nbands=1,
                experimental=dict(reuse_wfs_method='paw' if reuse else None),
                kpts=[[-0.3, 0.4, 0.2]],
                symmetry='off',
                mixer=Mixer(0.7, 5, 50.0))
    atoms.calc = calc

    first_iter_err = []

    def monitor():
        logerr = np.log10(calc.wfs.eigensolver.error)
        n = calc.scf.niter
        if n == 1:
            first_iter_err.append(logerr)

        if world.rank == 0:
            print('iter', n, 'err', logerr)
    calc.attach(monitor, 1)

    atoms.get_potential_energy()
    logerr1 = first_iter_err.pop()

    atoms.positions[:, 2] -= 2 * dz
    atoms.get_potential_energy()

    logerr2 = first_iter_err.pop()

    if world.rank == 0:
        print('reuse={}'.format(bool(reuse)))
        print('logerr1', logerr1)
        print('logerr2', logerr2)
        gain = logerr2 - logerr1
        print('gain', gain)
    return logerr2


noreuse_logerr = check(0)
reuse_logerr = check(1)
# Ref values: logerr=-3.6 without reuse_wfs and -5.0 with reuse_wfs
assert reuse_logerr < -4.8, reuse_logerr
assert reuse_logerr < noreuse_logerr - 1.2, (reuse_logerr, noreuse_logerr)



from ase import Atoms
from gpaw import GPAW
from gpaw.wavefunctions.pw import PW
from gpaw.xc.fxc import FXCCorrelation
from gpaw.test import equal
from gpaw.mpi import world

if world.size == 1:
    scalapack1 = None
    scalapack2 = None
elif world.size == 2:
    scalapack1 = (2, world.size // 2, 32)
    scalapack2 = None
else:
    scalapack1 = (2, world.size // 2, 32)
    scalapack2 = (2, world.size // 4, 32)

# H2
H2 = Atoms('H2', [(0, 0, 0), (0, 0, 0.7413)])
H2.set_pbc(True)
H2.set_cell((2., 2., 3.))
H2.center()
calc = GPAW(mode=PW(210, force_complex_dtype=True),
            eigensolver='rmm-diis',
            xc='LDA',
            basis='dzp',
            nbands=8,
            parallel={'domain': 1},
            convergence={'density': 1.e-6})
H2.set_calculator(calc)
H2.get_potential_energy()
calc.diagonalize_full_hamiltonian(nbands=80, scalapack=scalapack1)
calc.write('H2.gpw', mode='all')

ralda = FXCCorrelation('H2.gpw', xc='rALDA', nblocks=min(4, world.size))
E_ralda_H2 = ralda.calculate(ecut=[200])

rapbe = FXCCorrelation('H2.gpw', xc='rAPBE')
E_rapbe_H2 = rapbe.calculate(ecut=[200])

# H
H = Atoms('H')
H.set_pbc(True)
H.set_cell((2., 2., 3.))
H.center()
calc = GPAW(mode=PW(210, force_complex_dtype=True),
            eigensolver='rmm-diis',
            xc='LDA',
            basis='dzp',
            nbands=4,
            hund=True,
            parallel={'domain': 1},
            convergence={'density': 1.e-6})
H.set_calculator(calc)
H.get_potential_energy()
calc.diagonalize_full_hamiltonian(nbands=80, scalapack=scalapack2)
calc.write('H.gpw', mode='all')

ralda = FXCCorrelation('H.gpw', xc='rALDA')
E_ralda_H = ralda.calculate(ecut=[200])

rapbe = FXCCorrelation('H.gpw', xc='rAPBE', nblocks=min(4, world.size))
E_rapbe_H = rapbe.calculate(ecut=[200])

equal(E_ralda_H2, -0.8411, 0.001)
equal(E_ralda_H, 0.0029, 0.0001)
equal(E_rapbe_H2, -0.7233, 0.001)
equal(E_rapbe_H, 0.0161, 0.0001)



from ase import Atoms

from gpaw import GPAW, FermiDirac
from gpaw.test import equal

h = 0.4
q = 3
spin = True

s = Atoms('Fe')
s.center(vacuum=2.5)
convergence = {'eigenstates': 0.01, 'density': 0.1, 'energy': 0.1}

# use Hunds rules

c = GPAW(charge=q, h=h, nbands=5,
         hund=True,
         eigensolver='rmm-diis',
         occupations=FermiDirac(width=0.1),
         convergence=convergence)
s.calc = c
equal(s.get_magnetic_moment(), 5, 0.1)

# set magnetic moment

mm = [5]
s.set_initial_magnetic_moments(mm)
c = GPAW(charge=q, h=h, nbands=5,
         occupations=FermiDirac(width=0.1, fixmagmom=True),
         convergence=convergence)
s.calc = c
equal(s.get_magnetic_moment(), 5, 0.1)



from ase import Atoms
from ase.build import molecule
from gpaw import GPAW, PW
from gpaw.xc.fxc import FXCCorrelation
from gpaw.test import equal
from gpaw.mpi import world

if world.size == 1:
    scalapack1 = None
    scalapack2 = None
elif world.size == 2:
    scalapack1 = (2, world.size // 2, 32)
    scalapack2 = None
else:
    scalapack1 = (2, world.size // 2, 32)
    scalapack2 = (2, world.size // 4, 32)

# N2
N2 = molecule('N2')
N2.set_cell((2.5, 2.5, 3.5))
N2.center()
calc = GPAW(mode=PW(force_complex_dtype=True),
            eigensolver='rmm-diis',
            xc='LDA',
            nbands=16,
            basis='dzp',
            parallel={'domain': 1},
            convergence={'density': 1.e-6})
N2.set_calculator(calc)
N2.get_potential_energy()
calc.diagonalize_full_hamiltonian(nbands=80, scalapack=scalapack1)
calc.write('N2.gpw', mode='all')

ralda = FXCCorrelation('N2.gpw', xc='rALDA')
Ec_N2 = ralda.calculate(ecut=[50])

# N
N = Atoms('N', [(0, 0, 0)])
N.set_cell((2.5, 2.5, 3.5))
N.center()
calc = GPAW(mode=PW(force_complex_dtype=True),
            eigensolver='rmm-diis',
            xc='LDA',
            basis='dzp',
            nbands=8,
            hund=True,
            parallel={'domain': 1},
            convergence={'density': 1.e-6})
N.set_calculator(calc)
N.get_potential_energy()
calc.diagonalize_full_hamiltonian(nbands=80, scalapack=scalapack2)
calc.write('N.gpw', mode='all')

ralda = FXCCorrelation('N.gpw', xc='rALDA')
Ec_N = ralda.calculate(ecut=[50])

equal(Ec_N2, -6.17, 0.01,)
equal(Ec_N, -1.1085, 0.001)



import os

import numpy as np

from ase import Atoms
from gpaw import GPAW, PW
from gpaw.mpi import world
from gpaw.test import equal, findpeak
from gpaw.utilities import compiled_with_sl
from gpaw.response.df import DielectricFunction

# Comparing the plasmon peaks found in bulk sodium for two different
# atomic structures. Testing for idential plasmon peaks. Not using
# physical sodium cell.

a = 4.23 / 2.0
a1 = Atoms('Na',
           scaled_positions=[[0, 0.1, 0]],
           cell=(a, a, a),
           pbc=True)

# Expanding along x-direction
a2 = Atoms('Na2',
           scaled_positions=[[0, 0.1, 0], [0.5, 0.1, 0]],
           cell=(2 * a, a, a),
           pbc=True)

a1.calc = GPAW(gpts=(10, 10, 10),
               experimental={'niter_fixdensity': 2},
               mode=PW(300),
               kpts={'size': (8, 8, 8), 'gamma': True},
               parallel={'band': 1},
               #txt='small.txt',
               )

# Kpoint sampling should be halved in the expanded direction.
a2.calc = GPAW(gpts=(20, 10, 10),
               experimental={'niter_fixdensity': 2},
               mode=PW(300),
               kpts={'size': (4, 8, 8), 'gamma': True},
               parallel={'band': 1},
               #txt='large.txt',
               )

a1.get_potential_energy()
a2.get_potential_energy()

# Use twice as many bands for expanded structure
a1.calc.diagonalize_full_hamiltonian(nbands=20)
a2.calc.diagonalize_full_hamiltonian(nbands=40)

a1.calc.write('gs_Na_small.gpw', 'all')
a2.calc.write('gs_Na_large.gpw', 'all')

# Settings that should yield the same result
settings = [{'disable_point_group': True, 'disable_time_reversal': True},
            {'disable_point_group': False, 'disable_time_reversal': True},
            {'disable_point_group': True, 'disable_time_reversal': False},
            {'disable_point_group': False, 'disable_time_reversal': False}]

# Test block parallelization (needs scalapack)
if world.size > 1 and compiled_with_sl():
    settings.append({'disable_point_group': False,
                     'disable_time_reversal': False,
                     'nblocks': 2})

# Calculate the dielectric functions
dfs0 = []  # Arrays to check for self-consistency
dfs1 = []
dfs2 = []
dfs3 = []
dfs4 = []
dfs5 = []
for kwargs in settings:
    try:
        os.remove('chi0+0+0+0.pckl')
    except OSError:
        pass

    df1 = DielectricFunction('gs_Na_small.gpw',
                             domega0=0.03,
                             omegamax=10,
                             ecut=150,
                             name='chi0',
                             **kwargs)

    df1NLFCx, df1LFCx = df1.get_dielectric_function(direction='x')
    df1NLFCy, df1LFCy = df1.get_dielectric_function(direction='y')
    df1NLFCz, df1LFCz = df1.get_dielectric_function(direction='z')

    try:
        os.remove('chi1+0+0+0.pckl')
    except OSError:
        pass
    
    df2 = DielectricFunction('gs_Na_large.gpw',
                             domega0=0.03,
                             omegamax=10,
                             ecut=150,
                             name='chi1',
                             **kwargs)

    df2NLFCx, df2LFCx = df2.get_dielectric_function(direction='x')
    df2NLFCy, df2LFCy = df2.get_dielectric_function(direction='y')
    df2NLFCz, df2LFCz = df2.get_dielectric_function(direction='z')

    dfs0.append(df1NLFCx)
    dfs1.append(df1LFCx)
    dfs2.append(df1NLFCy)
    dfs3.append(df1LFCy)
    dfs4.append(df1NLFCz)
    dfs5.append(df1LFCz)

    # Compare plasmon frequencies and intensities
    w_w = df1.chi0.omega_w
    w1, I1 = findpeak(w_w, -(1. / df1LFCx).imag)
    w2, I2 = findpeak(w_w, -(1. / df2LFCx).imag)
    equal(w1, w2, 1e-2)
    equal(I1, I2, 1e-3)

    w1, I1 = findpeak(w_w, -(1. / df1LFCy).imag)
    w2, I2 = findpeak(w_w, -(1. / df2LFCy).imag)
    equal(w1, w2, 1e-2)
    equal(I1, I2, 1e-3)

    w1, I1 = findpeak(w_w, -(1. / df1LFCz).imag)
    w2, I2 = findpeak(w_w, -(1. / df2LFCz).imag)
    equal(w1, w2, 1e-2)
    equal(I1, I2, 1e-3)

# Check for self-consistency
for i, dfs in enumerate([dfs0, dfs1, dfs2, dfs3, dfs4, dfs5]):
    while len(dfs):
        df = dfs.pop()
        for df2 in dfs:
            try:
                assert np.max(np.abs((df - df2) / df)) < 1e-3
            except AssertionError:
                print(np.max(np.abs((df - df2) / df)))
                raise AssertionError



from ase.build import molecule
from gpaw import GPAW
import gpaw.dscf as dscf
from gpaw.test import equal

# Ground state calculation

calc = GPAW(mode='lcao',
            basis='dzp',
            nbands=8,
            h=0.2,
            xc='PBE',
            spinpol=True,
            convergence={'energy': 100,
                         'density': 1e-3,
                         'bands': -1})

CO = molecule('CO')
CO.center(vacuum=3)
CO.set_calculator(calc)

E_gs = CO.get_potential_energy()

# Excited state calculation

calc_es = GPAW(mode='lcao',
               basis='dzp',
               nbands=8,
               h=0.2,
               xc='PBE',
               spinpol=True,
               convergence={'energy': 100,
                            'density': 1e-3,
                            'bands': -1})

CO.set_calculator(calc_es)
lumo = dscf.MolecularOrbital(calc,
                             weights={0: [0, 0, 0,  1], 1: [0, 0, 0, -1]})
dscf.dscf_calculation(calc_es, [[1.0, lumo, 1]], CO)

E_es = CO.get_potential_energy()
dE = E_es - E_gs
print(dE)
equal(dE, 5.7595110076, 0.011)




from __future__ import print_function
import numpy as np
from ase.units import Hartree
from gpaw.occupations import FermiDirac, MethfesselPaxton, occupation_numbers


class KPoint:
    eps_n = np.empty(1)
    f_n = np.empty(1)
    weight = 0.2
    s = 0


k = KPoint()


def f(occ, x):
    k.eps_n[0] = x
    n, dnde, x, S = occ.distribution(k, 0.0)
    return n, dnde, S


def test(occ):
    print(occ)
    for e in [-0.3 / Hartree, 0, 0.1 / Hartree, 1.2 / Hartree]:
        n0, d0, S0 = f(occ, e)
        x = 0.000001
        np, dp, Sp = f(occ, e + x)
        nm, dm, Sm = f(occ, e - x)
        d = -(np - nm) / (2 * x)
        dS = Sm - Sp
        dn = np - nm
        print(d - d0, dS - e * dn)
        assert abs(d - d0) < 3e-5
        assert abs(dS - e * dn) < 1e-13


for w in [0.1, 0.5]:
    test(FermiDirac(w))
    for n in range(4):
        test(MethfesselPaxton(w, n))


occ = {'name': 'fermi-dirac', 'width': 0.1}

for eps_skn, weight_k, n in [
    ([[[0.0, 1.0]]], [1.0], 2),
    ([[[0.0, 1.0]], [[0.0, 2.0]]], [1.0], 3),
    ([[[0.0, 1.0, 2.0], [0.0, 2.0, 2.0]]], [0.5, 0.5], 2)]:
    f_skn, fl, m, s = occupation_numbers(occ, eps_skn, weight_k, n)
    print(f_skn, fl, m, s)
    assert abs(f_skn.sum() - n) < 1e-14, f_skn



from __future__ import print_function
import sys
import re

import numpy as np

from ase.build import molecule
from ase.units import Hartree
from gpaw import GPAW
from gpaw.mpi import rank, world
from gpaw.test import equal
from gpaw.gauss import Gauss
from gpaw.lrtddft import LrTDDFT, photoabsorption_spectrum
from gpaw.lrtddft.kssingle import KSSingles
try:
    from StringIO import StringIO  # Python 2
except ImportError:
    from io import StringIO

L = 10.0
txt=None
txt='-'

N2 = molecule('N2')
N2.set_cell([L, L, L])
#N2.set_pbc(True)
N2.center()

#try:
#    calc = GPAW('N2_wfs.gpw',
#                txt=txt,
#                parallel={'domain': world.size})
#    calc.converge_wave_functions()
#except:
if 1:
    calc = GPAW(h=0.25,
                experimental={'niter_fixdensity': 2},
                nbands=-5,
                spinpol=True,
                xc='PBE',
                txt=txt,
                eigensolver='cg',
                parallel={'domain': world.size})
    N2.set_calculator(calc)
    E0 = N2.get_potential_energy()
    calc.write('N2_wfs.gpw', 'all')

# selections
for obj in [KSSingles, LrTDDFT]:
    # selection using state numbers
    el = obj(calc, istart=3, jend=6, txt=txt)
    if hasattr(obj, 'diagonalize'):
        el.diagonalize()
#    print "*************** obj, len(obj)", obj.__name__, len(el)
    assert len(el) == 8
    # selection using an energy range
    el = obj(calc, energy_range=8, txt=txt)
    if hasattr(obj, 'diagonalize'):
        el.diagonalize()
#    print "*************** obj, len(obj)", obj.__name__, len(el)
    assert len(el) == 4
    el = obj(calc, energy_range=11.5, txt=txt)
#    print "*************** obj, len(obj)", obj.__name__, len(el)
    if hasattr(obj, 'diagonalize'):
        el.diagonalize()
    assert len(el) == 18
    if hasattr(obj, 'diagonalize'):
        el.diagonalize(energy_range=8)
        assert len(el) == 4

lr = LrTDDFT(calc, nspins=2)
lr.write('lrtddft3.dat.gz')
lr.diagonalize()

world.barrier()

# This is done to test if writing and reading again yields the same result
lr2 = LrTDDFT('lrtddft3.dat.gz')
lr2.diagonalize()

# Unfortunately not all of the lrtddft code is parallel
if rank == 0:
    Epeak = 19.5# The peak we want to investigate (this is alone)
    Elist = np.asarray([lrsingle.get_energy() * Hartree for lrsingle in lr])
    n = np.argmin(np.abs(Elist - Epeak)) # Index of the peak

    E = lr[n].get_energy() * Hartree
    osz = lr[n].get_oscillator_strength()
    print('Original object        :', E, osz[0])

    # Test the output of analyse
    origstdout = sys.stdout
    sys.stdout = sio = StringIO()
    lr.analyse(n)
    s = sio.getvalue()
    sys.stdout = origstdout
    match = re.findall(r'%i: E=([0-9]*\.[0-9]*) eV, f=([0-9]*\.[0-9]*)*' % n, s)
    Eanalyse = float(match[0][0])
    oszanalyse = float(match[0][1])
    print('From analyse           :', Eanalyse, oszanalyse)
    equal(E, Eanalyse, 1e-3)            # Written precision in analyse
    equal(osz[0], oszanalyse, 1e-3)

    E2 = lr2[n].get_energy() * Hartree
    osz2 = lr2[n].get_oscillator_strength()
    print('Written and read object:', E2, osz2[0])
    
    # Compare values of original and written/read objects
    equal(E, E2, 1e-4)
    for i in range(len(osz)):
        equal(osz[i], osz2[i], 1.7e-4)

    width = 0.05
    photoabsorption_spectrum(lr,
                             spectrum_file = 'lrtddft3-spectrum.dat',
                             width = width)
    # We need to be able to check the heights in the spectrum
    weight = Gauss(width).get(0)

    spectrum = np.loadtxt('lrtddft3-spectrum.dat', usecols = (0, 1))
    idx = (spectrum[:, 0] >= E - 0.1) & (spectrum[:, 0] <= E + 0.1)
    peak = np.argmax(spectrum[idx, 1]) + np.nonzero(idx)[0][0]
    Espec = spectrum[peak, 0]
    oszspec = spectrum[peak, 1] / weight

    print('Values from spectrum   :', Espec, oszspec)
    # Compare calculated values with values written to file
    equal(E, Espec, 1e-2)           # The spectrum has a low sampling
    equal(osz[0], oszspec, 1e-2)
